import os

from .. import param

def data_path():
    """
    Data path
    """
    path = os.path.join(os.path.dirname(__file__), '../data/')
    return path


def test_param_file():
    """
    Read Param file
    """
    param_file = os.path.join(data_path(), 'zphot.param.default')
    
    # No filename, read default
    pfile1 = param.EazyParam(PARAM_FILE=None)
    
    # Read filename
    pfile2 = param.EazyParam(PARAM_FILE=param_file)
    
    assert(pfile1['Z_MIN'] == pfile2['Z_MIN'])
    
    # Set new parameter
    pfile1['XPARAM'] = 1.
    assert('XPARAM' in pfile1.param_names)
    assert(pfile1['XPARAM'] == 1)




def build_matrices():
    """
    Build matrices for Blanton & Roweis optimization
    """ 
    #import unicorn
    import numpy as np
    #import br07
    import glob
    #from threedhst import catIO
    import matplotlib.pyplot as plt
    
    from astropy.table import Table
    import eazy
    
    from eazy.utils import running_median, nmad
    
    # UltraVista
    self = eazy.photoz.PhotoZ(param_file='zphot.param.new', translate_file='zphot.translate.new', zeropoint_file='zphot.zeropoint.FSPS_full')
    zbest = np.load('zbest_Nov3.npy')[0]
    self.best_fit(zbest=zbest)
    
    # FAST
    fout = Table.read('UVISTA_DR3_master_v1.1.fout', format='ascii.commented_header')
    
    ## RF fluxes
    if False:
        rf_tempfilt, f_rest = self.rest_frame_fluxes(f_numbers=[153,155,161], pad_width=0.1, percentiles=[2.5,16,50,84,97.5])
        np.save('uvista_rest_frame.npy', [rf_tempfilt, f_rest, idx])
    else:
        rf_tempfilt, f_rest, idx0 = np.load('uvista_rest_frame.npy')
    
    uv_data = -2.5*np.log10(f_rest[:, 0, 2] / f_rest[:, 1, 2])
    vj_data = -2.5*np.log10(f_rest[:, 1, 2] / f_rest[:, 2, 2])
    
    u_0 = rf_tempfilt.tempfilt[:,0] / rf_tempfilt.tempfilt[:,2]
    v_0 = rf_tempfilt.tempfilt[:,1] / rf_tempfilt.tempfilt[:,2]
    j_0 = rf_tempfilt.tempfilt[:,2] / rf_tempfilt.tempfilt[:,2]
        
    uv_temp = -2.5*np.log10(u_0 / v_0)
    vj_temp = -2.5*np.log10(v_0 / j_0)
    
    ## Template data
    lines = open('templates/fsps_full/tweak_spectra.param').readlines()
    templ_names = [templ.name.split('.dat')[0] for templ in self.templates]
    data = []
    for line in lines:
        if line.startswith('# '):
            data.append(line.split()[1:])
    columns = ['id', 'label', 'age', 'mass', 'sfr', 'Av', 'Ha_flux', 'Ha_cont']
    templ_data = Table(data=np.array(data), names=columns, dtype=[int, str, float, float, float, float, float, float])
    
    zsp_only=True
    
    zp_ix = self.f_numbers == self.param['PRIOR_FILTER']
    mag = self.param['PRIOR_ABZP'] - np.squeeze(2.5*np.log10(self.fnu[:, zp_ix]))
    
    idx = (mag < 23) | ((self.cat['z_spec'] > 0) & np.isfinite(mag))
    #idx = (mag < 22) | ((self.cat['z_spec'] > 0) & np.isfinite(mag))
    if zsp_only:
        idx &= (self.cat['z_spec'] > 0)
        
    if 'USE' in self.cat.colnames:
        # UltraVISTA
        use = (self.cat['USE'] == 1) & (self.cat['star'] == 0)
        use = (self.cat['contamination'] == 0) & (self.cat['nan_contam'] == 0) & (self.cat['star'] == 0)
    else:
        use = (self.cat['use'] == 1) & (self.cat['star_flag'] != 1)
        
    idx &= use
    
    idx_zsp = np.arange(self.NOBJ)[idx]
        
    ############# Refine photo-zs
    iter = 200
    iter = 300 # use Nov4 12 templates
    error_residuals, update_templates, update_zeropoints = True, False, True
    for i in range(6):
        self.iterate_zp_templates(idx=idx_zsp, update_templates=(i > 2), iter=iter+i, error_residuals=True, update_zeropoints=True, save_templates=(i > 2), n_proc=6)
        self.write_zeropoint_file('zphot.zeropoint.FSPS_full_12')
        #np.save('zbest.npy', [self.zbest])
    
    #############
    ## Fit full sample without updating zeropoints, etc.
    zp_ix = self.f_numbers == self.param['PRIOR_FILTER']
    mag = self.param['PRIOR_ABZP'] - np.squeeze(2.5*np.log10(self.fnu[:, zp_ix]))
    
    idx = (mag < 23) | ((self.cat['z_spec'] > 0) & np.isfinite(mag))
    if 'USE' in self.cat.colnames:
        # UltraVISTA
        use = (self.cat['USE'] == 1) & (self.cat['star'] == 0)
        use = (self.cat['contamination'] == 0) & (self.cat['nan_contam'] == 0) & (self.cat['star'] == 0)
    else:
        use = (self.cat['use'] == 1) & (self.cat['star_flag'] != 1)
        
    idx &= use
    
    ## Minimum number of OK data
    nfilt = ((self.fnu > 0) & (self.efnu > 0) & (self.fnu/self.efnu > 3)).sum(axis=1)
    idx &= nfilt > 6
    
    # reasonable colors
    idx &= np.isfinite(uv_data) & np.isfinite(vj_data) & (uv_data < 3) & (uv_data > -0.4) & (vj_data > -0.4) & (vj_data < 2.7) #quiescent
    
    #idx &= (fout['lssfr'] < -11) & (fout['Av'] < 0.5) & (vj_data > 0.5) & np.isfinite(uv_data) & np.isfinite(vj_data) & (uv_data < 2.4) #quiescent
    
    idx_full = np.arange(self.NOBJ)[idx]
    idx = idx_full
    
    ## Refit last photo-zs
    if False:
        self.iterate_zp_templates(idx=idx, update_templates=False, iter=206, error_residuals=True, update_zeropoints=False, save_templates=False, n_proc=6)
        self.iterate_zp_templates(idx=idx, update_templates=False, iter=207, error_residuals=True, update_zeropoints=False, save_templates=False, n_proc=6)
        np.save('zbest_Nov3.npy', [self.zbest])
    
    ##############
    ## Build NMF matrix
    izbest = np.asarray(np.round(np.interp(self.zbest, self.tempfilt.zgrid, np.arange(self.NZ))),dtype=int)
    
    NOBJ = len(idx)
    data = np.zeros((NOBJ, self.NFILT, self.NZ))
    data_ivar = np.zeros((NOBJ, self.NFILT, self.NZ))
    
    for i in range(len(idx)):
        print(i)
        ix = idx[i]
        iz = izbest[ix]
        
        flu = self.fnu[ix,:]*self.zp*self.ext_corr
        ivar = 1/((self.efnu[ix,:]*self.zp*self.ext_corr)**2 + (0.02*flu)**2)
        
        ok_i = (self.fnu[ix,:] > 0) & (self.efnu[ix,:] > 0)
        data[i, ok_i, iz] = flu[ok_i]
        data_ivar[i, ok_i, iz] = ivar[ok_i]
            
    ### Show UVJ
    if False:
        
        if True:
            plt.scatter(vj_data[idx], uv_data[idx], c=fout['lssfr'][idx], vmin=-12, vmax=-8, marker='s', alpha=0.03, cmap='jet_r', edgecolor='None')
            #plt.scatter(vj_data[idx], uv_data[idx], c=fout['lmass'][idx], vmin=9, vmax=11, marker='s', alpha=0.03, cmap='jet_r', edgecolor='None')

        else:
            plt.scatter(vj_data[idx], uv_data[idx], color='k', marker='.', alpha=0.03)
        
        templ_ssfr = np.log10(templ_data['sfr']/templ_data['mass'])
        plt.scatter(vj_temp, uv_temp, c=templ_ssfr, vmin=-12, vmax=-8, marker='s', alpha=0.8, edgecolor='None', cmap='jet_r')
        plt.xlim(-0.3, 3.5); plt.ylim(0,3.5)
        
    ### Generate NMF components
    # Normalize template array by J band
    M = np.zeros((self.NTEMP, self.NFILT, self.NZ))
    for i in range(self.NTEMP):
        M[i,:,:] = self.tempfilt.tempfilt[:,i,:].T / rf_tempfilt.tempfilt[i,2]

    ok_temp = templ_data['Av'] > -1
    n_components = 12
    
    ## Fulls set, 12 templates, coeffs_blue
    n_components = 8
    ok_temp = ~((templ_ssfr < -10) & (templ_data['Av'] <= 1))
    
    # Quiescent templates, 4 templates, coeffs_red
    if True:
        ok_temp = (templ_ssfr < -10) & (templ_data['Av'] <= 1)
        n_components = 4

    M = M[ok_temp, :, :]    
    ds = data.shape
    ms = M.shape
        
    delta = ms[0] // n_components+1
    
    coeffs = np.ones((n_components, ms[0]))*1./delta/(n_components*5)
    for i in range(n_components):
        coeffs[i, i*delta:(i+1)*delta] = 1./delta
    
    so = np.argsort(vj_temp[ok_temp])
    coeffs[:,so] = coeffs*1
    
    coeffsx = np.ones((n_components, self.NTEMP))*1./delta/(n_components*5)
    coeffsx[:, ok_temp] += coeffs
    
    ## Split on quiescent templates for initialization
    coeffs = np.vstack((coeffs_red, coeffs_blue))
    
    templates = np.ones((ds[0], n_components))
    
    ### Loop so you can break in and still have the coeffs updated.  
    ### total of 4000 iters
    plt.scatter(vj_data[idx], uv_data[idx], color='k', marker='.', alpha=0.03)
    #plt.scatter(vj_temp, uv_temp, color='r', marker='.', alpha=0.8)
    plt.xlim(-0.3, 3.5); plt.ylim(0,3.5)
    
    plt.scatter(-2.5*np.log10(np.dot(coeffs, v_0[ok_temp])/np.dot(coeffs, j_0[ok_temp])), -2.5*np.log10(np.dot(coeffs, u_0[ok_temp])/np.dot(coeffs, v_0[ok_temp])), color='orange', alpha=0.5, s=40, marker='s')
    
    from templates import nmf_sparse
    
    for i in range(1, 100):
        print('Restart #{0}'.format(i))
        coeffs, templates = nmf_sparse(data.reshape((ds[0],-1)), data_ivar.reshape((ds[0],-1)), M.reshape((ms[0],-1)), n_components=n_components, NITER=40, tol_limit=1.e-5, coeffs=coeffs*1, templates=templates*1.)
        
        np.save('full_nmf_coeffs.npy', [[i], idx, coeffs, templates])
        
        if True:
            plt.scatter(-2.5*np.log10(np.dot(coeffs, v_0[ok_temp])/np.dot(coeffs, j_0[ok_temp])), -2.5*np.log10(np.dot(coeffs, u_0[ok_temp])/np.dot(coeffs, v_0[ok_temp])), color='r', alpha=0.3, s=40, marker='.')
            plt.savefig('full_nmf_iter_{0:03d}.png'.format(i))
    
    ##################################
    ### Best combined templates
    i, idx0, coeffs, templates = np.load('full_nmf_coeffs.npy')
    #i, idx0, coeffs, templates = np.load('nmf_coeffs.npy')
    templ_array = np.array([templ.flux for templ in self.templates])
        
    vj0 = -2.5*np.log10(np.dot(coeffs, v_0)/np.dot(coeffs, j_0))
    uv0 = -2.5*np.log10(np.dot(coeffs, u_0)/np.dot(coeffs, v_0))
    
    plt.scatter(vj_data[idx], uv_data[idx], color='k', marker='.', alpha=0.03)
    plt.xlim(-0.4, 2.9); plt.ylim(-0.4,2.9)
    #plt.scatter(vj0, uv0, color='r', marker='s')
    
    # Put normalization back in 
    coeffs_norm = coeffs/rf_tempfilt.tempfilt[:,2]
    coeffs_norm = (coeffs_norm.T / coeffs_norm.sum(axis=1)).T
    
    # V-band
    nmf_norm = np.dot(coeffs_norm, rf_tempfilt.tempfilt[:,1])
    coeffs_norm = (coeffs_norm.T / nmf_norm).T
    
    nmf_templates = np.dot(coeffs_norm, templ_array).T
    
    ## Effective Av
    alpha = 10**(-0.4*templ_data['Av'])
    fv = np.dot(coeffs_norm, rf_tempfilt.tempfilt[:,1])
    fv_corr = np.dot(coeffs_norm, rf_tempfilt.tempfilt[:,1]/alpha)
    Av = -2.5*np.log10(fv/fv_corr)
    
    ## Other parameters
    mass = np.dot(coeffs_norm, templ_data['mass'])
    sfr = np.dot(coeffs_norm, templ_data['sfr'])
    ssfr = sfr/mass
    
    Ha_flux = np.dot(coeffs_norm, templ_data['Ha_flux'])
    Ha_cont = np.dot(coeffs_norm, templ_data['Ha_cont'])
    
    Ha_EW = Ha_flux / Ha_cont
    
    NTEMP = coeffs.shape[0]
    
    nu_v = 3.e8/5000.e-10
    Lv = fv*nu_v
    
    fp = open('templates/uvista_nmf/spectra_12_Nov4.param','w')
    fp.write('## i age mass sfr Av Ha_flux Ha_cont fnu_v\n')
    for i in range(NTEMP):
        temp_file = 'nmf_fsps_Nov4_{0:02d}.dat'.format(i+1)
        label = '# {id:2d} -1  {mass:5.4e}  {sfr:.3e}  {Av:.3f}  {Ha_flux:.3e}  {Ha_cont:.3e}  {fnu:.3e}'.format(id=i+1, mass=mass[i], sfr=sfr[i], Av=Av[i], Ha_flux=Ha_flux[i], Ha_cont=Ha_cont[i], fnu=fv[i])
        print(label)
        fp.write(label+'\n')
        fp.write('{0:2d}  templates/uvista_nmf/{1}  1.0 0 1.0\n'.format(i+1, temp_file))
        
        fpt = open('./templates/uvista_nmf/{0}'.format(temp_file), 'w')
        fpt.write('#  i age mass sfr Av Ha_flux Ha_cont fnu_v\n')
        fpt.write(label+'\n')
        fpt.close()
        
        fpt = open('./templates/uvista_nmf/{0}'.format(temp_file), 'ab')
        np.savetxt(fpt, np.array([self.templates[0].wave, nmf_templates[:,i]]).T, fmt='%.6e')
        fpt.close()
        
    fp.close()
    
    ## Show the templates in UVJ: sSFR, MLv, Av, Ha EW
    sh = 0.8
    fig = plt.figure(figsize=[11,8])
    
    ax = fig.add_subplot(221)
    ax.scatter(vj_data[idx], uv_data[idx], color='k', marker='.', alpha=0.01)
    ax.set_xlim(-0.4, 2.9); ax.set_ylim(-0.4,2.9)
    sc = ax.scatter(vj0, uv0, marker='s', c=np.log10(ssfr), s=100, vmin=-12, vmax=-8, cmap='jet_r')
    cb = plt.colorbar(sc, ax=ax, shrink=sh)
    cb.set_label('log sSFR')
    for i in range(NTEMP):
        ax.text(vj0[i], uv0[i]+0.1, '{0:d}'.format(i+1), ha='center', va='bottom', size=8, backgroundcolor='w')
        
    ax.grid()
    
    ax = fig.add_subplot(222)
    ax.scatter(vj_data[idx], uv_data[idx], color='k', marker='.', alpha=0.01)
    ax.set_xlim(-0.4, 2.9); ax.set_ylim(-0.4,2.9)
    sc = ax.scatter(vj0, uv0, marker='s', c=np.log10(mass/Lv), s=100, vmin=-1, vmax=1.5, cmap='jet')
    cb = plt.colorbar(sc, ax=ax, shrink=sh)
    cb.set_label('log M/Lv')
    ax.grid()
    
    ax = fig.add_subplot(223)
    ax.scatter(vj_data[idx], uv_data[idx], color='k', marker='.', alpha=0.01)
    ax.set_xlim(-0.4, 2.9); ax.set_ylim(-0.4,2.9)
    sc = ax.scatter(vj0, uv0, marker='s', c=Av, s=100, vmin=0, vmax=3, cmap='jet')
    cb = plt.colorbar(sc, ax=ax, shrink=sh)
    cb.set_label('Av')
    ax.grid()
    
    ax = fig.add_subplot(224)
    ax.scatter(vj_data[idx], uv_data[idx], color='k', marker='.', alpha=0.01)
    ax.set_xlim(-0.4, 2.9); ax.set_ylim(-0.4,2.9)
    sc = ax.scatter(vj0, uv0, marker='s', c=np.log10(Ha_flux/Ha_cont), s=100, vmin=1, vmax=3, cmap='jet_r')
    cb = plt.colorbar(sc, ax=ax, shrink=sh)
    cb.set_label(r'log H$\alpha$ EW')
    ax.grid()
    
    fig.tight_layout(pad=0.4)
    fig.savefig('spectra_12_Nov4.png')
    
    fig = plt.figure(figsize=[8,8])
    for i in range(NTEMP):
        ax = fig.add_subplot(3,4,1+i)
        ax.plot(self.templates[0].wave[::10], nmf_templates[::10,:]/(3.e18/5500.**2), color='k', alpha=0.1)
        ax.plot(self.templates[0].wave, nmf_templates[:,i]/(3.e18/5500.**2), color='k', alpha=0.8, linewidth=2)
        ax.semilogx()
        ax.set_xlim(1000,5.e4)
        ax.set_ylim(0,10)
        if i < 8:
            ax.set_xticklabels([])
        
        if (i % 4) > 0: 
            ax.set_yticklabels([])
        
        ax.text(0.9, 0.9, '#{0}'.format(i+1), transform=ax.transAxes, ha='right', va='top', size=12)
        
    fig.tight_layout(pad=0.1)
    fig.savefig('spectra_12_Nov4_SED.png')
    
def check_parameters():
    """
    Redshifts & SP parameters from the new NMF fits
    """
    self = eazy.photoz.PhotoZ(param_file='zphot.param.new', translate_file='zphot.translate.new', zeropoint_file='zphot.zeropoint.FSPS_full_12')
    
    zp_ix = self.f_numbers == self.param['PRIOR_FILTER']
    mag = self.param['PRIOR_ABZP'] - np.squeeze(2.5*np.log10(self.fnu[:, zp_ix]))
    
    idx = (mag < 23) | ((self.cat['z_spec'] > 0) & np.isfinite(mag))
    if 'USE' in self.cat.colnames:
        # UltraVISTA
        use = (self.cat['USE'] == 1) & (self.cat['star'] == 0)
        use = (self.cat['contamination'] == 0) & (self.cat['nan_contam'] == 0) & (self.cat['star'] == 0)
    else:
        use = (self.cat['use'] == 1) & (self.cat['star_flag'] != 1)
        
    idx &= use
    
    ## Minimum number of OK data
    nfilt = ((self.fnu > 0) & (self.efnu > 0) & (self.fnu/self.efnu > 3)).sum(axis=1)
    idx &= nfilt > 6
    
    #idx &= (fout['lssfr'] < -11) & (fout['Av'] < 0.5) & (vj_data > 0.5) & np.isfinite(uv_data) & np.isfinite(vj_data) & (uv_data < 2.4) #quiescent
    
    idx_full = np.arange(self.NOBJ)[idx]
    idx = idx_full
    
    ## Refit last photo-zs
    if True:
        self.iterate_zp_templates(idx=idx, update_templates=False, iter=401, error_residuals=True, update_zeropoints=False, save_templates=False, n_proc=6)
        self.iterate_zp_templates(idx=idx, update_templates=False, iter=402, error_residuals=True, update_zeropoints=False, save_templates=False, n_proc=6)
        np.save('zbest_Nov4_12.npy', [self.zbest])
    
    rf_tempfilt, f_rest = self.rest_frame_fluxes(f_numbers=[153,155,161], pad_width=0.1, percentiles=[2.5,16,50,84,97.5])
    
############################################
# def old_build_matrices(NTEMP=5, field='cosmos', hmax=24, zmax=2.5):
#     """
#     Build matrices for Blanton & Roweis optimization
#     """ 
#     #import unicorn
#     import numpy as np
#     #import br07
#     import glob
#     #from threedhst import catIO
#     from astropy.table import Table
#     import eazy
#     
#     from eazy.utils import running_median, nmad
#     
#     # UltraVista
#     self = eazy.photoz.PhotoZ(param_file='zphot.param.new', translate_file='zphot.translate.new', zeropoint_file='zphot.zeropoint.new')
#     zbest = np.load('zbest.npy')[0]
#     self.best_fit(zbest=zbest)
#     
#     zsp_only=False
#     
#     zp_ix = self.f_numbers == self.param['PRIOR_FILTER']
#     mag = self.param['PRIOR_ABZP'] - np.squeeze(2.5*np.log10(self.fnu[:, zp_ix]))
#     
#     idx = (mag < 23) | ((self.cat['z_spec'] > 0) & np.isfinite(mag))
#     #idx = (mag < 22) | ((self.cat['z_spec'] > 0) & np.isfinite(mag))
#     if zsp_only:
#         idx &= (self.cat['z_spec'] > 0)
#         
#     if 'USE' in self.cat.colnames:
#         # UltraVISTA
#         use = (self.cat['USE'] == 1) & (self.cat['star'] == 0)
#         use = (self.cat['contamination'] == 0) & (self.cat['nan_contam'] == 0) & (self.cat['star'] == 0)
#     else:
#         use = (self.cat['use'] == 1) & (self.cat['star_flag'] != 1)
#         
#     idx &= use
#     
#     idx = np.arange(self.NOBJ)[idx]
#     
#     iter = 200
#     error_residuals, update_templates, update_zeropoints = True, False, True
#     
#     for i in range(3):
#         self.iterate_zp_templates(idx=idx, update_templates=update_templates, iter=iter+i, error_residuals=error_residuals, update_zeropoints=update_zeropoints, save_templates=True, n_proc=6)
#         self.write_zeropoint_file('zphot.zeropoint.FSPS_full')
#         np.save('zbest.npy', [self.zbest])
#         
#     ### Fit all objects    
#     idx_full = np.arange(self.NOBJ)
#     error_residuals, update_templates, update_zeropoints = True, False, False
#     iter = 99
#     self.iterate_zp_templates(idx=idx_full, update_templates=update_templates, iter=iter, error_residuals=error_residuals, update_zeropoints=update_zeropoints, save_templates=False, n_proc=6)
#     
#     ### Bin on U-V vs sSFR
#     plt.scatter(np.log10(ssfr[idx]), uv[idx], c=vj[idx], vmin=-0.5, vmax=2, alpha=0.2, edgecolor='0.5')
#     
#     xh, yh = np.clip(np.log10(ssfr[idx]), -12.95, -8.05), np.clip(uv[idx], 0.01, 2.49)
#     h2 = np.histogram2d(xh, yh, range=[(-13,-8), (0,2.5)], bins=[8,8])
# 
#     xh, yh = np.clip(vj[idx], 0.01, 2.99), np.clip(uv[idx], 0.01, 3.49)
#     h2 = np.histogram2d(xh, yh, range=[(0,3), (0,3.5)], bins=[16,16])
#     
#     ## Binned rest-frame SEDs
#     #full_templates = self.param.read_templates(templates_file='templates/fsps_full/tweak_spectra.param')
#     full_templates = self.param.read_templates(templates_file='templates/fsps_full/spectra.param')
#     full_tempfilt = eazy.photoz.TemplateGrid(self.zgrid, full_templates, self.param['FILTERS_RES'], self.f_numbers, add_igm=True, galactic_ebv=self.param.params['MW_EBV'], Eb=self.param['SCALE_2175_BUMP'], n_proc=0)
#     full_rf_tempfilt = eazy.photoz.TemplateGrid(np.array([0,0.1]), full_templates, self.param['FILTERS_RES'], np.array([153,155,161]), add_igm=False, galactic_ebv=0, Eb=self.param['SCALE_2175_BUMP'], n_proc=-1)
#     full_rf_tempfilt.tempfilt = np.squeeze(full_rf_tempfilt.tempfilt[0,:,:])
#         
#     ix = np.argmax(h2[0])
#     j, i = np.unravel_index(ix, h2[0].shape)
#     
#     Q = ssfr[idx] > -10
#     testQ = Q
#     
#     sel = (xh >= h2[1][j]) & (xh <= h2[1][j+1]) & (yh >= h2[2][i]) & (yh <= h2[2][i+1]) & testQ
#     
#     fnu_corr = self.fnu[idx[sel],:]*self.zp*self.ext_corr
#     efnu_corr = self.efnu[idx[sel],:]*self.zp*self.ext_corr
# 
#     #irest = 1
#     fnu_corr = (fnu_corr.T/f_rest[idx[sel],irest,2]).T
#     efnu_corr = (efnu_corr.T/f_rest[idx[sel],irest,2]).T
#     lc_rest = (self.lc[:,np.newaxis]/(1+self.zbest[idx[sel]])).T
#     
#     templ = np.zeros((sel.sum(), self.NFILT, full_tempfilt.NTEMP))
#     
#     import specutils.extinction
#     import astropy.units as u
#     f99 = specutils.extinction.ExtinctionF99(a_v = self.tempfilt.galactic_ebv * 3.1)
#     fred = 10**(-0.4*f99(full_rf_tempfilt.lc[irest]*(1+self.zbest[idx][sel])*u.AA))
#     
#     for ii in range(sel.sum()):
#         zi = self.zbest[idx][sel][ii]
#         templ[ii,:,:] = (full_tempfilt(zi).T/(full_rf_tempfilt.tempfilt[:,irest]*fred[ii]))
#     
#     be = efnu_corr.flatten()
#     A = templ.reshape((-1,full_tempfilt.NTEMP))
#     b = fnu_corr.flatten()/be
#     
#     ok = (self.fnu[idx[sel],:].flatten() > -99) & (self.efnu[idx[sel],:].flatten() > 0) & (lc_rest.flatten() > 1300)
# 
#     sh = self.fnu[idx[sel],:].shape
#     oksh = ok.reshape(sh)
#     
#     yy = (fnu_corr/self.ext_corr)/(lc_rest/rf_tempfilt.lc[irest])**2
#     xm, ym, ys, N = running_median(lc_rest[oksh], yy[oksh], use_median=True, use_nmad=True, NBIN=100)
#     ym_i = np.interp(lc_rest, xm, ym)
#     ys_i = np.interp(lc_rest, xm, ys)
#     oksh &= np.abs(ym_i - yy) < 3*ys_i
#     ok = oksh.flatten()
#     
#     coeffs, resid = scipy.optimize.nnls((A[ok,:].T/be[ok]).T,b[ok])
#     # coeffs, resid, rank, s = np.linalg.lstsq((A[ok,:].T/be[ok]).T,b[ok])
#     # 
#     # amatrix = unicorn.utils_c.prepare_nmf_amatrix(be[ok]**2, A[ok,:].T)
#     # coeffs_nmf = unicorn.utils_c.run_nmf(fnu_corr.flatten()[ok], be[ok]**2, A[ok,:].T, amatrix, verbose=True, toler=1.e-5)
#     
#     best = np.dot(A, coeffs)
# 
#     yym = (best.reshape(sh)/self.ext_corr)/(lc_rest/rf_tempfilt.lc[irest])**2
#     xmm, ymm, ysm, Nm = running_median(lc_rest[oksh], yym[oksh], use_median=True, use_nmad=True, NBIN=100)
#     
#     if False:
#         plt.scatter(lc_rest[oksh], (fnu_corr/self.ext_corr)[oksh]/(lc_rest[oksh]/rf_tempfilt.lc[irest])**2, alpha=0.05*100/sel.sum(), color='k', marker='.')
#         plt.scatter(lc_rest[oksh], (best.reshape(sh)/self.ext_corr)[oksh]/(lc_rest[oksh]/rf_tempfilt.lc[irest])**2, alpha=0.05*100/sel.sum(), color='r', marker='.', zorder=2)
#    
#     plt.errorbar(xm, ym, ys, color='k')
#     plt.plot(xmm, ymm, color='r', marker='.', alpha=0.4)
#     plt.xlim(800,1.e5); plt.ylim(0.01,10); log()
#     
#     tf = np.array([full_templates[ii].flux / (full_rf_tempfilt.tempfilt[ii, irest]*3.e18/full_rf_tempfilt.lc[irest]**2) for ii in range(full_tempfilt.NTEMP)])
#     tt = np.dot(coeffs, tf)
#     plt.plot(full_templates[0].wave, tt, color='r')
#     
#     plt.scatter(lc_rest[oksh], fnu_corr[oksh]/best[ok], alpha=0.05*100/sel.sum(), color='k', marker='.')
#     plt.xlim(800,1.e5); plt.ylim(0.5,1.5); plt.semilogx()
    
    


import numpy as np

from .. import utils

def test_milky_way():
    """
    Test that milky way extinction is avaliable
    """
    import astropy.units as u
    
    f99 = utils.GalacticExtinction(EBV=1./3.1, Rv=3.1)
    
    # Out of range, test that at least run
    _ = f99(10)
    _ = f99(5.e4)
    _ = f99(10.e4)
    
    # Data types
    np.testing.assert_allclose(f99(5500), 1., rtol=0.05, atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)

    np.testing.assert_allclose(f99(5500.), 1., rtol=0.05, atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)
    
    np.testing.assert_allclose(f99(5500.*u.Angstrom), 1., rtol=0.05, 
                               atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)
    
    np.testing.assert_allclose(f99(0.55*u.micron), 1., rtol=0.05, 
                               atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)

    np.testing.assert_allclose(f99(100*u.micron), 0., rtol=0.05, 
                               atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)
                                   
    # Arrays
    np.testing.assert_allclose(f99([5500., 5500.]), 1., rtol=0.05, 
                               atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)
    
    arr = np.ones(10)*5500.                            
    np.testing.assert_allclose(f99(arr), 1., rtol=0.05, 
                               atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)

    np.testing.assert_allclose(f99(arr*u.Angstrom), 1., rtol=0.05, 
                               atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)

    np.testing.assert_allclose(f99(arr*u.Angstrom), 1., rtol=0.05, 
                               atol=0.05,
                               equal_nan=False, err_msg='', verbose=True)
                               

def test_interp_conserve():
    """
    Test interpolation function
    """
    # High-frequence sine function, should integrate to zero
    ph = 0.1
    xp = np.arange(-np.pi,3*np.pi,0.001)
    fp = np.sin(xp/ph)
    fp[(xp <= 0) | (xp > 2*np.pi)] = 0

    x = np.arange(-np.pi/2,2.5*np.pi,1)

    y1 = utils.interp_conserve(x, xp, fp)
    integral = np.trapz(y1, x)
    
    np.testing.assert_allclose(integral, 0., rtol=1e-04, atol=1.e-4,
                               equal_nan=False, err_msg='', verbose=True)


def test_log_zgrid():
    """
    Test log_zgrid function
    """
    ref = np.array([0.1, 0.21568801, 0.34354303, 0.48484469, 
                    0.64100717, 0.8135934])
    
    vals = utils.log_zgrid(zr=[0.1, 1], dz=0.1)
    np.testing.assert_allclose(vals, ref, rtol=1e-04, atol=1.e-4,
                               equal_nan=False, err_msg='', verbose=True)


def test_invert():
    """
    Test matrix invert helper
    """        
    a = np.array([[1., 2.], [3., 4.]])
    ainv = utils.safe_invert(a)
    assert(np.allclose(np.dot(a, ainv), np.eye(2)))


def test_query_string():
    """
    """
    ra, dec = 53.14474, -27.78552
    
    qstr = utils.query_html(ra, dec, with_coords=True, replace_comma=False, 
                            queries=['CDS'])
    
    expected = '(53.144740, -27.785520) <a href="http://vizier.u-strasbg.fr/viz-bin/VizieR?-c=53.14474+%2D27.78552&-c.rs=1.0">CDS</a>'
    assert(qstr == expected)
                    
    for co in [True, False]:
        for re in [True, False]:
            qstr = utils.query_html(ra, dec, with_coords=co, replace_comma=re, 
                            queries=['CDS','ESO','MAST','ALMA', 'LEG','HSC'])
                            
    


"""
Making templates with SPS tools
"""

def agn_templates():
    
    import fsps
    import eazy
    import matplotlib.pyplot as plt
    
    sp = fsps.StellarPopulation()
    res = eazy.filters.FilterFile(path=None)  
    jfilt = res[161]
    
    sp.params['dust_type'] = 2
    sp.params['dust2'] = 0
    
    sp.params['agn_tau'] = 10
    sp.params['fagn'] = 0
    wave, f0 = sp.get_spectrum(tage=0.5, peraa=True)

    fig = plt.figure(figsize=(6,4))
    ax = fig.add_subplot(111)
    
    sp.params['fagn'] = 1.    
    for tau in [5,10,20,50,80,120]:
        sp.params['agn_tau'] = tau
        wave, f1 = sp.get_spectrum(tage=0.5, peraa=True)
        
        label = 'agn_tau{0:03d}'.format(sp.params['agn_tau'])
        ax.plot(wave/1.e4, f1-f0, label=label, alpha=0.5)
        
        #fnorm = np.trapz(f1-f0, wave)
        templ = templates.Template(arrays=(wave, f1-f0), name=label, meta={'agn_tau':sp.params['agn_tau']})
        fnorm = templ.integrate_filter(jfilt, flam=True)/jfilt.pivot
        templ.flux *= fnorm
        
        file = 'templates/agn_tau/{0}.fits'.format(templ.name)
        print(tau, file)
        
        tab = templ.to_table() 
        tab.write(file, overwrite=True)
    
    ax.legend()
    ax.loglog()
    ax.grid()
    ax.set_ylim(1.e-7, 1.e-3)
    ax.set_xlim(90/1.e4, 200)
    
    ax.set_xlabel(r'$\lambda$, $\mu$m')
    ax.set_ylabel(r'$f_\lambda$')
    
    fig.tight_layout(pad=0.1)
    fig.savefig('templates/agn_tau/fsps_agn.png')
    
def extreme_starburst():
    """
    Make extreme starburst template
    """
    import eazy
    import grizli.utils
    
    sps = None
    
    #self = sps.ExtendedFsps(zmet=10, zcontinuous=False, add_neb_emission=True, sfh=4, tau=0.2)
    self = sps.ExtendedFsps(logzsol=0, zcontinuous=True, add_neb_emission=True, sfh=4, tau=0.2)

    self._set_extend_attrs(line_sigma=100, lya_sigma=300)
    #self._set_extend_attrs(line_sigma=50, lya_sigma=200)
    #self._set_extend_attrs(line_sigma=400, lya_sigma=400)
    
    self.set_fir_template()
    self.set_dust()
    
    self.params['tau'] = 0.2
    
    path = 'templates/fsps_full_2019/'
    par = grizli.utils.read_catalog(path+'xfsps_QSF_12_v3.param.fits')
    files = glob.glob(path+'xfsps_QSF_12_v3_0??.dat')
    files.sort()
    par['file'] = files
    
    Av = 5
    templ = self.get_full_spectrum(tage=0.3, Av=Av, set_all_templates=True)
    
    res = eazy.filters.FilterFile(os.getenv('EAZYCODE') + 'filters/FILTER.RES.latest') 
    
    vfilt = res.filters[154]
    jfilt = res.filters[160]
    Lv = templ.integrate_filter(vfilt, flam=False)*3.e18/vfilt.pivot
    jflux = templ.integrate_filter(jfilt, flam=True)
    
    # Replace 6 template in param.fits
    tab = templ.to_table()
    tab['flux'] /= jflux*jfilt.pivot
    plt.plot(tab['wave'], tab['flux'], alpha=0.5)

    if False:
        plt.plot(self.templ_unred.wave, self.templ_unred.flux/(jflux*jfilt.pivot), alpha=0.5)
        plt.plot(self.templ_orig.wave, self.templ_orig.flux/(jflux*jfilt.pivot), alpha=0.5)
    
    # Replace the dusty-old template with this starburst
    file = 'templates/fsps_full_2019/{0}.fits'.format(templ.name)
    tab.write(file, overwrite=True)
    
    par['file'][5] = os.path.basename(file)
    par['mass'][5] = tab.meta['stellar_mass']
    par['energy_abs'][5] = tab.meta['energy_absorbed']
    par['Lv'][5] = Lv
    par['Av'][5] = tab.meta['Av']
    par['sfr'][5] = tab.meta['sfr']
    #par['sfr100'][5] = tab.meta['sfr100']
    par['LIR'][5] = tab.meta['energy_absorbed']
        
    par.write(path+'xfsps_QSF_12_v3.SB.param.fits', overwrite=True)
    
    # Blue template without Lyman alpha
    
    ex = sps.ExtendedFsps(logzsol=-1, zcontinuous=True, add_neb_emission=True, sfh=4, tau=0.2)
    ex.set_fir_template()
    
    ex.params['logzsol'] = -1
    ex.params['gas_logz'] = -1
    ex.params['gas_logu'] = -2
    ex.params['tau'] = 0.3
    tage, Av = 0.2, 0.03

    templ = ex.get_full_spectrum(tage=tage, Av=Av, scale_lyman_series=0.1, set_all_templates=False)
    
    vfilt = res.filters[154]
    jfilt = res.filters[160]
    Lv = templ.integrate_filter(vfilt, flam=False)*3.e18/vfilt.pivot
    jflux = templ.integrate_filter(jfilt, flam=True)
    
    # Replace 6 template in param.fits
    tab = templ.to_table()
    tab['flux'] /= jflux*jfilt.pivot
    plt.plot(tab['wave'], tab['flux'])
    plt.loglog()
    
    # Replace the blue template with this one
    file = 'templates/fsps_full_2019/{0}.fits'.format(templ.name)
    tab.write(file, overwrite=True)
    
    path = 'templates/fsps_full_2019/'
    par = grizli.utils.read_catalog(path+'xfsps_QSF_12_v3.SB.param.fits')
    
    ix = 6
    par['file'][ix] = os.path.basename(file)
    par['mass'][ix] = tab.meta['stellar_mass']
    par['energy_abs'][ix] = tab.meta['energy_absorbed']
    par['Lv'][ix] = Lv
    par['Av'][ix] = tab.meta['Av']
    par['sfr'][ix] = tab.meta['sfr']
    par['LIR'][ix] = tab.meta['energy_absorbed']
        
    par.write(path+'xfsps_QSF_12_v3.SB.param.fits', overwrite=True)
    
    # Remove Ly-alpha 
    for file in par['file']:
        if file.endswith('dat'):
            file = file.replace('v3_', 'v3_gm09_')
            print(file)
            templ = grizli.utils.read_catalog(file)
            lya = np.abs(templ['wave'] - 1216) < 70
            interp = np.interp(templ['wave'][lya], templ['wave'][~lya], templ['flux'][~lya])
            templ['flux'][lya] = interp
            templ.write(file.replace('v3_', 'v3_nolya_'), format='ascii.commented_header', overwrite=True)
    
def spline_sfh():
    """
    Generate a template set with spline-basis SFHs
    """
    import os
    
    import numpy as np
    import matplotlib.pyplot as plt
    
    from grizli.utils import bspline_templates
    import eazy.sps
    
    # Grid encompassing FSPS.log_age    
    step = 0.022
    ages = 10**np.arange(np.log10(3.e-4), 
                          np.log10(14.12)+step, step)
    
    ages = np.arange(3.e-4, 14.127, 0.001)
    
    # Spline range between tmin, tmax
    tmin, tmax, Ns = 0.02, 8, 7
    
    # For Grid
    tmin, tmax, Ns = 0.03, 9, 21
    
    spl = bspline_templates(ages, get_matrix=True, 
                            log=True, df=Ns, clip=0.,
                            minmax=(np.log(tmin), np.log(tmax)))
    
    Na = len(ages)
    
    # Log-normals
    centers = [0.03, 0.1, 0.3, 0.8, 1.4, 3., 8.]
    widths = [0.21, 0.21, 0.21, 0.08, 0.08, 0.21, 0.21]
    
    centers = [0.03, 0.10, 0.25, 0.50, 0.8, 1.6, 4.]
    widths =  [0.17, 0.17, 0.17, 0.09, 0.09, 0.17, 0.17]
    
    # For grid
    if 0:
        centers = [0.03, 0.065, 0.1, 0.2, 0.3, 0.55, 0.8, 1.1, 1.4, 2.2, 3., 5., 8.]
        widths = np.diff(np.log10(centers))/2.35
        widths = np.append([widths[0]], widths)
    
    Ng = len(centers)
    gau = np.zeros((Na, Ng))    
    for i in range(Ng):
        log_age = np.log10(ages/centers[i])
        w = widths[i]
        gau[:,i] = 1./np.sqrt(2*np.pi*w**2)*np.exp(-log_age**2/2/w**2)
    
    spl = gau    
        
    age100 = ages <= 0.1
    
    sp = eazy.sps.ExtendedFsps(logzsol=0, zcontinuous=True, add_neb_emission=True, sfh=4)
    
    sp.set_fir_template()
    #sp.set_dust(dust_obj_type='KC13')    
    #sp.params['dust_index'] = 0. #-0.3
    sp.set_dust(dust_obj_type='WG00x')
    sp.params['imf_type'] = 1.
    sp.get_spectrum(tage=0.1)
    
    N = spl.shape[1]
    vband = ["v"]
    ssfr = np.zeros(N)

    logz = np.array([-1., -0.5, 0., 0., 0., 0., 0.])
    logu = np.array([-1.5, -2.0, -2.5, -2.5, -2.5, -2.5, -2.5])
    dusts = [[0.02, 0.5],
             [0.02, 0.5, 1.0, 2.0, 3.0, 5.],
             [0.02, 1., 2.0],
             [0.02, 1., 2.0],
             [0.02], 
             [0.02],
             [0.02]]
    
    # for i in [0,1,2,3,4]:
    #     dusts[i] = list(np.arange(0.2, 3.05-0.25*i, 0.1))
    
    # Full grid
    if N > 7:
        dusts = [[0.02, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 4., 5., 6., 7., 8., 10., 12]]*N
        logz = np.zeros(N)
        logu = np.zeros(N)-2.5
        logz[:2] = -1.
        logu[:2] = -1.5

    bands = ['u','v','2mass_j', 'i1500', 'i2800']
    uvj_mags = np.zeros((N, len(bands)))
    band_ages = np.zeros((N, len(bands)))
    
    _ = sp.get_spectrum()  
    fsps_ages = 10**(sp.log_age-9)
    from grizli.utils_c.interp import interp_conserve_c
    
    norm_sfh = spl.T*0.
    ysfh = np.ones((N, len(fsps_ages)))
    
    # 30 Myr
    t0 = 0.03
    end_age = np.zeros(N)
    end_clip = 1.e-3
    
    sfh_interp = norm_sfh*0.
    
    for i in range(N):
        sfh = spl.T[i,:]*1.
        sfh_t0 = np.interp(t0, ages, sfh)
        #sfh[ages < t0] = sfh_t0
        sfh_i = sfh/np.trapz(sfh, ages*1.e9) 
        
        # Evaluate tage where sfr is down by some factor from the max        
        if sfh[-1]/sfh.max() > end_clip:
            tstart = ages[-1]
        else:
            tstart = ages[sfh/sfh.max() > end_clip].max()
        
        end_age[i] = tstart
        #end_age[i] = 14.12 #ages.max()
        
        # Constant SFR below t0 (30 Myr)
        sfr0 = np.interp(t0, ages, sfh_i)
        sfh_i[ages < t0] = sfr0
        sfr_min = sfh_i.max()/1000.
        imax = np.argmax(sfh_i)
        sfh_i[(sfh_i < sfr_min) & (ages < ages[imax])] = sfr_min
        
        if 1:
            # Now evaluate SFH in "forward" direction,rather than lookback
            tforward = end_age[i]-ages[::-1]
            clip = tforward > 0
            sfh_i[~clip[::-1]] = sfr_min
            norm_sfh[i,:] = sfh_i*1.
                
            sfh = np.interp(ages, tforward, norm_sfh[i,::-1], left=0., right=sfr0) #, integrate=0)
            sfh[sfh == 0] = sfr_min
            sfh_interp[i,:] = sfh
            plt.plot(ages, sfh)
        else:
            sfh_interp[i,:] = sfh_i[::-1]
            norm_sfh[i,:] = sfh_i*1.
            plt.plot(ages, sfh_i)
        
        sp.params['sfh'] = 3
        sp.set_tabular_sfh(ages, sfh_interp[i,:]) 
        sp.params['logzsol'] = logz[i]
        sp.params['gas_logz'] = logz[i]
        sp.params['gas_logu'] = logu[i]
    
        sp.params['compute_light_ages'] = True
        band_ages[i,:] = sp.get_mags(tage=end_age[i], bands=bands)

        sp.params['compute_light_ages'] = False
        uvj_mags[i,:] = sp.get_mags(tage=end_age[i], bands=bands)
        ssfr[i] = sp.sfr100/sp.stellar_mass
    
    if False:
        #plt.figure()
        plt.plot(uvj_mags[:,1] - uvj_mags[:,2], uvj_mags[:,0]-uvj_mags[:,1], color='w', alpha=1., zorder=100)
        plt.plot(uvj_mags[:,1] - uvj_mags[:,2], uvj_mags[:,0]-uvj_mags[:,1], color='0.5', alpha=0.5, zorder=101)
        plt.scatter(uvj_mags[:,1] - uvj_mags[:,2], uvj_mags[:,0]-uvj_mags[:,1], c=np.log10(np.maximum(ssfr, 1.e-12)), vmin=-12, vmax=-8, zorder=1000, marker='s', s=100, edgecolor='w')
    
    
    ### Make all templates
    res = eazy.filters.FilterFile('FILTER.RES.latest')
    uvj_res = [res[153],res[155],res[161]]
    breakme=False
        
    templates = []
    sp.params['dust2'] = 0.
    sp.params['sfh'] = 3.
    sp.params['compute_light_ages'] = False
    kwargs = {'scale_lyman_series':0.1}
    
    plt.figure()
    
    for i in range(N):
        #sp.set_tabular_sfh(ages, sfh_interp[i,:]) 
        sp.set_tabular_sfh(ages, sfh_interp[i,:]) 
        
        sp.params['logzsol'] = logz[i]
        sp.params['gas_logz'] = logz[i]
        sp.params['gas_logu'] = logu[i]
        
        ssfr_i = sp.sfr100/sp.stellar_mass
        
        for Av in dusts[i]:
            templ = sp.get_full_spectrum(tage=end_age[i], Av=Av,
                                         get_template=True, 
                                         set_all_templates=False, **kwargs)
            
            templ.ageV = band_ages[i,1]
            jflux = templ.integrate_filter(uvj_res[2], flam=True)
            templates.append(templ)
            plt.plot(templ.wave, templ.flux/jflux, label=templ.name, alpha=0.8)
        
        if (i == 0) & breakme:
            break
    
    plt.loglog()
    
    fig = plt.figure()
    plt.scatter(vj[sel], uv[sel], c=np.log10(zout['SFR']/zout['mass'])[sel], vmin=-12, vmax=-8) 
    uvj = np.array([templ.integrate_filter_list(uvj_res) for templ in templates])
    ssfr_temp = np.array([templ.meta['sfr100']/templ.meta['stellar_mass'] for templ in templates])
    plt.scatter(-2.5*np.log10(uvj[:,1]/uvj[:,2]), -2.5*np.log10(uvj[:,0]/uvj[:,1]), c=np.log10(ssfr_temp), vmin=-12, vmax=-8, marker='s', edgecolor='w', s=100)
    
    # Try NMF decomposition
    if False:
        from sklearn.decomposition import NMF
        model = NMF(n_components=5, init='nndsvda', random_state=0, verbose=True, solver='cd', tol=1.e-6, alpha=0.01, beta_loss='frobenius', max_iter=100000)
        clip = (templ.wave > 2500) & (templ.wave < 1.6e4)
        X = [templ.flux/(uvj[i,1]*3.e18/5500**2) for i, templ in enumerate(templates)]
        dust_array = np.hstack(dusts)
        ix = np.array([1,3,4,5,6,9,10,12,13]) 
        ix = np.where((dust_array > 0.1) & (dust_array < 4))[0]
    
        W = model.fit_transform(np.array(X)[ix,clip].T)
        H = model.components_
        uvjr = H.dot((uvj.T/uvj[:,1]).T[ix,:])

        plt.scatter(-2.5*np.log10(uvj[ix,1]/uvj[ix,2]), -2.5*np.log10(uvj[ix,0]/uvj[ix,1]), c=np.log10(ssfr_temp)[ix], vmin=-12, vmax=-8, marker='s', edgecolor='r', s=100)
        plt.scatter(-2.5*np.log10(uvjr[:,1]/uvjr[:,2]), -2.5*np.log10(uvjr[:,0]/uvjr[:,1]), vmin=-12, vmax=-8, marker='v', edgecolor='w', s=100, alpha=0.8)
    
    # Write templates
    if N > 8:
        param_file = 'templates/spline_templates/spline.grid.param'
    else:
        param_file = 'templates/spline_templates/spline.param'

    fp = open(param_file,'w')
    for i, templ in enumerate(templates):
        tab = templ.to_table()
        tab.meta['ageV'] = templ.ageV
        name = 'spline_age{0:4.2f}_av{1:3.1f}'.format(tab.meta['ageV'], tab.meta['Av'])
        line = f'{i+1} templates/spline_templates/{name}.fits 1.0'
        print(line)
        fp.write(line+'\n')
        tab.write(f'templates/spline_templates/{name}.fits', overwrite=True)
    
    fp.close()
    
    # Metadata
    cols = ('file','Av','mass','Lv','sfr','LIR','energy_abs','ageV')
    rows = []
    for i, templ in enumerate(templates):
        vflux = templ.integrate_filter(uvj_res[1], flam=False)
        Lv = vflux*3.e18/uvj_res[1].pivot
        tab = templ.to_table()
        name = 'spline_age{0:4.2f}_av{1:3.1f}'.format(templ.ageV, templ.meta['Av'])
        row = [name, templ.meta['Av'], templ.meta['stellar_mass'], Lv, templ.meta['sfr100'], templ.meta['energy_absorbed'], templ.meta['energy_absorbed'], templ.ageV]
        rows.append(row)
    
    par = utils.GTable(names=cols, rows=rows)
    for line in ['Ha','O3','Hb','O2','Lya']:
        par['line_flux_'+line] = 0.
        par['line_C_'+line] = 0.
        par['line_EW_'+line] = 0.
        
    par.write(param_file+'.fits', overwrite=True)
    
    # color space spanned by random draws
    NT = len(templates)
    rnd = 10**(np.random.rand(1000*NT, NT)-3)
    for i in range(NT):
        rnd[i*1000:(i+1)*1000,i] = 1.
        
    uvjr = rnd.dot((uvj.T/uvj[:,0]).T)
    plt.scatter(-2.5*np.log10(uvjr[:,1]/uvjr[:,2]), -2.5*np.log10(uvjr[:,0]/uvjr[:,1]), c='k', marker='.', alpha=0.1)

    # Interpolation not working?
    for i in range(N):
        sp.set_tabular_sfh(ages, sfh_interp[i,:]) 

        #sp.set_tabular_sfh(fsps_ages, ysfh[i,:])
        _ = sp.get_spectrum()
    
        if 0:
            # Show in lookback time
            plt.plot(ages, norm_sfh[i,:], color='r', alpha=0.5)
            plt.plot(end_age[i]-10**(sp.log_age-9), sp.sfr, color='k', alpha=0.5) 
            plt.plot(end_age[i]-10**(sp.log_age-9), ysfh[i,:], color='g', alpha=0.5) 
            plt.vlines(end_age[i], 1.e-30, 10, color='b', alpha=0.5)
            sp.get_spectrum(tage=end_age[i], zmet=None)
            plt.scatter(0.01, sp.sfr, marker='o', color='b')
        else:
            _ = sp.get_spectrum()
            plt.plot(10**(sp.log_age-9), sp.sfr, color='k', alpha=0.5) 
            plt.plot(end_age[i]-ages, norm_sfh[i,:], color='r', alpha=0.5)
            plt.vlines(end_age[i], 1.e-30, 10, color='b', alpha=0.5)
            sp.get_spectrum(tage=end_age[i], zmet=None)
            plt.scatter(end_age[i], sp.sfr, marker='o', color='b')
        
            #_ = sp.get_spectrum()
            #plt.plot(10**(sp.log_age-9), sp.sfr/np.interp(end_age[i], fsps_ages, sp.sfr)*np.interp(end_age[i], end_age[i]-ages[::-1], norm_sfh[i,::-1]), alpha=0.5) 
        
    #####################
    # Do light-weighted ages add linearly? - YES!
    ij = [1,3]
    coeffs = np.array([0.2, 0.5])
 
    sfh = sfh_i*0.   
    for k in range(len(ij)):
        i = ij[k]
        sfh += coeffs[k]*spl.T[i,:]/np.trapz(spl.T[i,:], ages*1.e9) 
    
    sp.set_tabular_sfh(13-ages[::-1], sfh[::-1]) 
    sp.params['compute_light_ages'] = True
    combined_age = sp.get_mags(tage=13, bands=vband)
    
    vflux = 10**(-0.4*uvj_mags[ij,1])
    c_norm = (coeffs*vflux); c_norm /= c_norm.sum() 
    coeffs_age = vband_ages[ij].dot(c_norm)
    print(combined_age, coeffs_age)
    
    
    
def fit_dust_wg00():
    """
    Fit WG00 as flexible model
    """
    from dust_attenuation import averages, shapes, radiative_transfer
    from astropy.modeling.fitting import LevMarLSQFitter
    #from importlib import reload
    #reload(averages); reload(shapes)
    #reload(averages); reload(shapes)
    
    shapes.x_range_N09 = [0.01, 1000] 
    averages.x_range_C00 = [0.01, 1000]
    averages.x_range_L02 = [0.01, 0.18]

    tau_V = 1.0
    tau_V_grid = np.array([0.25, 0.5, 0.75, 1.0, 1.5, 2.0, 2.5, 3.0,
                           3.5, 4.0, 4.5, 5.0, 5.5, 6.0, 7.0, 8.0,
                           9.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0,
                           40.0, 45.0, 50.0])
    
    tau_V_grid = tau_V_grid[tau_V_grid < 11]
    
    i=-1
    
    params = []
    for i in range(len(tau_V_grid)):
        tau_V = tau_V_grid[i]                      
        wg00 = radiative_transfer.WG00(tau_V=tau_V, **WG00_DEFAULTS)
    
        model = shapes.N09()
        model.fixed['x0'] = True
    
        fitter = LevMarLSQFitter()
    
        wave = np.logspace(np.log10(0.18), np.log10(3), 100)
        y = wg00(wave*u.micron)
        _fit = fitter(model, wave, y)
    
        plt.plot(wave, y, label='tau_V = {0:.2f}'.format(tau_V), color='k', alpha=0.4)
        
        #plt.plot(wave, _fit(wave))

        shapes.x_range_N09 = [0.01, 1000] 
        averages.x_range_C00 = [0.01, 1000]
    
        wfull = np.logspace(-1.5, np.log10(20.1), 10000)
        plt.plot(wfull, _fit(wfull), color='r', alpha=0.4)
        params.append(_fit.parameters)
    
    params = np.array(params)
    N = params.shape[1]
    vx = np.linspace(0, 10, 100)
    order = [3, 5, 5, 5, 5]
    
    coeffs = {}
    
    for i in range(N):
        fig = plt.figure()
        ax = fig.add_subplot(111)
        ax.plot(tau_V_grid, params[:,i], marker='o', label=_fit.param_names[i])
        
        c = np.polyfit(tau_V_grid, params[:,i], order[i])
        ax.plot(vx, np.polyval(c, vx))
        ax.legend()
        ax.grid()
        coeffs[_fit.param_names[i]] = c

def test_fit_phot():

    import matplotlib.pyplot as plt
    import eazy.sps
    # dummy vars
    ids = []
    i = -1
    
    sp = eazy.sps.ExtendedFsps(logzsol=0, zcontinuous=True, add_neb_emission=True, sfh=4)
    sp.set_fir_template()
    #sp.set_dust(dust_obj_type='KC13')    
    #sp.params['dust_index'] = 0. #-0.3
    sp.set_dust(dust_obj_type='WG00x')
    sp.params['imf_type'] = 1.
    __ = sp.get_full_spectrum(tage=0.1)
    
    sp.set_lognormal_sfh(lognorm_center=-1, lognorm_logwidth=0.1, verbose=True) 
    sp.params['sfh'] = 3
    sp.is_lognorm_sfh = True
    sp.set_lognormal_sfh(lognorm_center=-1, lognorm_logwidth=0.1, verbose=True) 

    plist = ['tage','Av']
    func_kwargs = {'lorentz': False, 'emwave': [1216.0, 6564], 'scale_lyman_series': 0.0}
    lsq_kwargs = {'method': 'trf', 'max_nfev': 200, 'loss': 'huber', 'x_scale': 1.0, 'verbose': True, 'diff_step': [0.05, 0.05]}
    
    TEF = self.TEF
    
    i+=1
    id = ids[i]
    show_fnu=True
    zshow = None
    show_components = True
    maglim = (27.2,19.5)
    
    _ = self.show_fit(id, show_components=show_components, show_fnu=show_fnu, maglim=maglim, zshow=zshow, showpz=False)
    
    ax = _[0].axes[0]
    
    is_flam = (show_fnu < 1)
    
    z=_[1]['z']
    sp.params['zred'] = z
    eazy.sps.BOUNDS['zred'] = [z-0.2, z+0.2, 0.01]
    
    phot_dict = _[1]
    filters = self.filters
    
    _phot = sp.fit_phot(phot_dict, filters, plist=plist, func_kwargs=func_kwargs, verbose=True, lsq_kwargs=lsq_kwargs, show=ax, TEF=TEF)
    _phot['z'] = sp.params['zred']
    _phot['age'] = _phot['age_i1500']*1000
    for k in sp.meta:
        _phot[k] = sp.meta[k]
        
    label = 'logm={log_mass:.2f}\n lw_age={age:.1f} Myr\n Av={Av:.1f} \n logzsol={logzsol:.1f} gas_logz={gas_logz:.1f} gas_logu={gas_logu:.1f}'.format(**_phot)
    templ = _phot['templ']
    iz = templ.zindex(_phot['z'])
    
    igm = templ.igm_absorption(_phot['z'], scale_tau=1.4)
    ax.plot(templ.wave*(1+_phot['z'])/1.e4, templ.flux_fnu(iz)*_phot['scale']*igm, label=label, color='r', alpha=0.5)
    ax.legend()
    # ax.set_xlim(0.3, 3000)
    # ax.set_ylim(0.05, 5000)
    # if id == 319:
    #     ax.errorbar(1200., 460, 80, color='r', marker='o', alpha=0.5)
    # else:
    #     ax.errorbar(1200., 230, 80, color='r', marker='o', alpha=0.5)
    
    ax.set_ylim(-0.6, 10)  
    ax.set_xlim(0.45, 6)  
    _[0].tight_layout(pad=0.1)
    _[0].savefig(f'{root}_{id:05d}.fsps.png')



import os
import warnings

import numpy as np
import matplotlib.pyplot as plt

import astropy.stats
import astropy.units as u

CLIGHT = 299792458.0 # m/s

TRUE_VALUES = [True, 1, '1', 'True', 'TRUE', 'true', 'y', 'yes', 'Y', 'Yes']
FALSE_VALUES = [False, 0, '0', 'False', 'FALSE', 'false', 'n', 'no', 'N', 'No']

FNU_CGS = u.erg/u.second/u.cm**2/u.Hz
FLAM_CGS = u.erg/u.second/u.cm**2/u.Angstrom

DATA_PATH = None

def set_data_path(path='$EAZYCODE'):
    """
    Make symbolic links to EAZY inputs

    Parameters
    ----------
    path : str
        Full directory path or environment variable pointing to the old eazy
        C-code repository that provides the template and filter files.

        If `path.startswith('$')` then treat path as an environment variable.

        If you install from the repository that provides the eazy-photozy
        code as a submodule, then you should be able to run with `path=None`
        and retrieve the files directly from the repository.  This should
        also work with the `pip` installation.

        Another safe way to ensure that the necessary files are avialable is
        to clone the `eazy-photoz` repository and set an environment variable
        to point to it (e.g, 'EAZYCODE'), which you then pass as the `path`
        argument.

    """
    global DATA_PATH
    if path.startswith('$'):
        path = os.getenv(path)

    if path is None:
        # Use the code attached to the repository
        path = os.path.join(os.path.dirname(__file__), 'data/')
    
    DATA_PATH = path
    return path
    
set_data_path()

def bool_param(value, false_values=FALSE_VALUES, true_values=TRUE_VALUES, which='false', check_both=True):
    """
    Flexible booleans
    
    If ``which == 'false'``, test that ``value not in false_values``.
    
    If ``which == 'true'``, test ``value in true_values``.
    
    If ``check_both`` and ``value`` isn't in either list, return the value 
    itself.
    
    """
    if which == 'false':
        test = value not in false_values
    elif which == 'true':
        test = value in true_values
    else:
        raise ValueError("Option ``which`` must be 'true' or 'false'")
    
    if check_both:
        if value not in true_values + false_values:
            test = value
            
    return test


def path_to_eazy_data():
    """
    Return internal path to ``eazy/data``.
    """
    return os.path.join(os.path.dirname(__file__), 'data')


def set_warnings(numpy_level='ignore', astropy_level='ignore'):
    """
    Set global numpy and astropy warnings
    
    Parameters
    ----------
    numpy_level : 'ignore', 'warn', 'raise', 'call', 'print', 'log'
        Numpy error level (see `numpy.seterr`).
        
    astropy_level : 'error', 'ignore', 'always', 'default', 'module', 'once'
        Astropy error level (see `warnings.simplefilter`).
    
    """
    from astropy.utils.exceptions import AstropyWarning
    
    np.seterr(all=numpy_level)
    warnings.simplefilter(astropy_level, category=AstropyWarning)


def running_median(xi, yi, NBIN=10, reverse=False, bins=None, x_func=np.median, y_func=np.median, std_func=astropy.stats.mad_std, x_kwargs={}, y_kwargs={}, std_kwargs={}, use_biweight=False, integrate=False, **kwargs):
    """
    Binned median/biweight/nmad statistics
    
    Parameters
    ----------
    xi : array-like
        Data of independent variable
    
    yi : array-like
        Data of dependent variable
    
    NBIN : int
        Number of bins along `xi`
    
    reverse : bool
        Calculate bins starting at largest values of `xi`
    
    bins : array-like
        Fixed bins, rather than calculating with `NBIN`
    
    x_func : function
        Function to compute moments of `xi`
    
    y_func, std_func : function
        Functions to compute moments of `yi`.  Assumed to be the central 
        value and dispersion, but don't have to be
    
    x_kwargs, y_kwargs, std_kwargs : dict
        Keyword arguments to pass to moment functions
    
    use_biweight : bool
        Use robust biweight estimators:
        
            - `x_func` : `astropy.stats.biweight_location`
            - `y_func` : `astropy.stats.biweight_location`
            - `std_func` : `astropy.stats.biweight_midvariance`
        
    integrate : bool
        Numerically integrate `yi` with the trapezoidal rule within the bins
    
    Returns
    -------
    xm, ym, ys : array-like
        Binned moments of `xi` and `yi`
    
    yn : array-like
        Number of entries per bin
        
    """
    
    NPER = xi.size // NBIN
    if bins is None:
        so = np.argsort(xi)
        if reverse:
            so = so[::-1]
        
        bx = np.linspace(0,len(xi),NBIN+1)
        bins = np.interp(bx, np.arange(len(xi)), xi[so])
        if reverse:
            bins = bins[::-1]
    
    NBIN = len(bins)-1
           
    xm = np.ones(NBIN)
    xs = np.zeros_like(xm)
    ym = np.zeros_like(xm)
    ys = np.zeros_like(xm)
    N = np.zeros(NBIN, dtype=int)
    
    if use_biweight:
        x_func = astropy.stats.biweight_location
        y_func = astropy.stats.biweight_location
        std_func = astropy.stats.biweight_midvariance
        
    #idx = np.arange(NPER, dtype=int)
    for i in range(NBIN):
        in_bin = (xi > bins[i]) & (xi <= bins[i+1])
        N[i] = in_bin.sum() #N[i] = xi[so][idx+NPER*i].size
        
        if integrate:
            xso = np.argsort(xi[in_bin])
            ma = xi[in_bin].max()
            mi = xi[in_bin].min()
            xm[i] = (ma+mi)/2.
            dx = (ma-mi)
            ym[i] = np.trapz(yi[in_bin][xso], xi[in_bin][xso])/dx
        else:
            xm[i] = x_func(xi[in_bin], **x_kwargs)
            ym[i] = y_func(yi[in_bin], **y_kwargs)
        
        ys[i] = std_func(yi[in_bin], **std_kwargs)
                    
    return xm, ym, ys, N


def nmad(data):
    """
    Normalized median absolute deviation statistic
    
    Just a wrapper around `astropy.stats.mad_std`.
    """
    import astropy.stats
    #return 1.48*astropy.stats.median_absolute_deviation(arr)
    return astropy.stats.mad_std(data)


def log_zgrid(zr=[0.7,3.4], dz=0.01):
    """Make a logarithmically spaced redshift grid
    
    Parameters
    ----------
    zr : [float, float]
        Minimum and maximum of the desired grid
    
    dz : float
        Step size, dz/(1+z)
    
    Returns
    -------
    zgrid : array-like
        Redshift grid
    
    """
    zgrid = np.exp(np.arange(np.log(1+zr[0]), np.log(1+zr[1]), dz))-1
    return zgrid


def trapz_dx(x):
    """
    Return trapezoid rule coefficients, useful for numerical integration 
    using a dot product
    
    Parameters
    ----------
    x : array-like
        Independent variable
    
    Returns
    -------
    dx : array_like
        Coefficients for trapezoidal rule integration.
    """
    dx = np.zeros_like(x)
    diff = np.diff(x)/2.
    dx[:-1] += diff
    dx[1:] += diff
    return dx


def clipLog(im, lexp=1000, cmap=[-1.4914, 0.6273], scale=[-0.1,10]):
    """
    Return normalized array like DS9 log
    """
    import numpy as np
    
    contrast, bias = cmap
    clip = (np.clip(im, scale[0], scale[1])-scale[0])/(scale[1]-scale[0])
    clip_log = np.clip((np.log10(lexp*clip+1)/np.log10(lexp)-bias)*contrast+0.5, 0, 1)
    
    return clip_log


def get_mw_dust(ra, dec, **kwargs):
    """
    Wrapper around functions to try to query for the MW E(B-V)
    """
    try:
        ebv = get_dustmaps_dust(ra, dec, web=True)
        return ebv
    except:
        pass
        
    try:
        ebv = get_dustmaps_dust(ra, dec, web=False)
        return ebv
    except:
        pass
    
    try:
        ebv = get_irsa_dust(ra, dec, **kwargs)
        return ebv
    except:
        pass
    
    return 0.00
    
def get_dustmaps_dust(ra, dec, web=True, **kwargs):
    "Use https://github.com/gregreen/dustmaps"
    
    from dustmaps.sfd import SFDQuery, SFDWebQuery
    from astropy.coordinates import SkyCoord
    
    coords = SkyCoord(ra, dec, unit='deg', frame='icrs')
    
    if web:
        sfd = SFDWebQuery()
    else:
        sfd = SFDQuery()
        
    ebv = sfd(coords)
    return ebv

def get_irsa_dust(ra=53.1227, dec=-27.805089, type='SandF'):
    """
    Get Galactic dust reddening from NED/IRSA at a given position
    http://irsa.ipac.caltech.edu/applications/DUST/docs/dustProgramInterface.html
    
    Parameters
    ----------
    ra, dec : float
        RA/Dec in decimal degrees.
        
    type : 'SFD', 'SandF'
        Dust model, with        
            SandF = Schlafly & Finkbeiner 2011 (ApJ 737, 103) 
              SFD = Schlegel et al. 1998 (ApJ 500, 525)
    
    Returns
    -------
    ebv : float
        Color excess E(B-V), in magnitudes
    
    """
    import os
    import tempfile   
    import urllib.request
    from astropy.table import Table
    from lxml import objectify
    
    query = 'http://irsa.ipac.caltech.edu/cgi-bin/DUST/nph-dust?locstr={0:.4f}+{1:.4f}+equ+j2000'.format(ra, dec)
    
    req = urllib.request.Request(query)
    response = urllib.request.urlopen(req)
    resp_text = response.read().decode('utf-8')
    
    root = objectify.fromstring(resp_text)
    stats = root.result.statistics

    if type == 'SFD':
        return float(str(stats.refPixelValueSFD).split()[0])
    else:
        return float(str(stats.refPixelValueSandF).split()[0])


def fill_between_steps(x, y, z, ax=None, *args, **kwargs):
    """
    Make `matplotlib.pyplot.fill_between` work like linestyle='steps-mid'.
    """
    so = np.argsort(x)
    mid = x[so][:-1] + np.diff(x[so])/2.
    xfull = np.append(np.append(x, mid), mid+np.diff(x[so])/1.e6)
    yfull = np.append(np.append(y, y[:-1]), y[1:])
    zfull = np.append(np.append(z, z[:-1]), z[1:])
    
    so = np.argsort(xfull)
    if ax is None:
        ax = plt.gca()
    
    ax.fill_between(xfull[so], yfull[so], zfull[so], *args, **kwargs)


def safe_invert(arr):
    """
    Version-safe matrix inversion using `numpy.linalg.inv` or `numpy.matrix.I`
    """
    try:
        from numpy.linalg import inv
        _inv = inv(arr)
    except:
        _inv = np.matrix(arr).I.A
    
    return _inv


class GalacticExtinction(object):
    def __init__(self, EBV=0, Rv=3.1, force=None, radec=None, ebv_type='SandF'):
        """
        Wrapper to use either `specutils.extinction` or the `extinction` 
        modules, which have different calling formats.  The results from 
        both of these modules should be equivalent.
                
        Parameters
        ----------
        EBV : float
            Galactic reddening, e.g., from `https://irsa.ipac.caltech.edu/applications/DUST/`.
            
        Rv : float
            Selective extinction ratio, `Rv=Av/(E(B-V))`.
        
        radec : None or (float, float)
            If provided, try to determine EBV based on these coordinates 
            with `get_irsa_dust(type=[ebv_type])` or `dustmaps`. 
            
        force : None, 'extinction', 'specutils.extinction'
            Force use one or the other modules.  If `None`, then first try
            to import `specutils.extinction` and if that fails use
            `extinction`.
        """
        import importlib
        
        # Import handler
        if force == 'specutils.extinction':
            import specutils.extinction
            self.module = 'specutils.extinction'
        elif force == 'extinction':
            from extinction import Fitzpatrick99
            self.module = 'extinction'
        elif force == 'dust_extinction':
            from dust_extinction.parameter_averages import F99
            self.module = 'dust_extinction'
        else:
            modules = [['dust_extinction.parameter_averages', 'F99'], 
                       ['extinction','Fitzpatrick99'],
                       ['specutils.extinction','ExtinctionF99']]
            
            self.module = None
            for (mod, cla) in modules:
                try:
                    _F99 = getattr(importlib.import_module(mod), cla)
                    self.module = mod
                    break
                except:
                    continue
            
            if self.module is None:
                raise ImportError("Couldn't import extinction module from "
                                  "dust_extinction, extinction or specutils") 
                                       
            # try:
            #     from specutils.extinction import ExtinctionF99
            #     self.module = 'specutils.extinction'
            # except:
            #     from extinction import Fitzpatrick99
            #     self.module = 'extinction'
        
        if radec is not None:
            self.EBV = get_mw_dust(ra=radec[0], dec=radec[1], type=ebv_type)
        else:    
            self.EBV = EBV
        
        self.Rv = Rv
        
        if self.module == 'dust_extinction.parameter_averages':
            self.f99 = _F99(Rv=self.Rv)
            
        elif self.module == 'specutils.extinction':
            self.f99 = _F99(self.Av)
            #self.Alambda = f99(self.wave*u.angstrom)
        else:
            self.f99 = _F99(self.Rv)
            #self.Alambda = f99(self.wave*u.angstrom, Av)
    
    @property
    def Av(self):
        return self.EBV*self.Rv
    
    @property    
    def info(self):
        msg = ('F99 extinction with `{0}`: Rv={1:.1f}, '
              'E(B-V)={2:.3f} (Av={3:.2f})')
        return msg.format(self.module, self.Rv, self.EBV, self.Av)
        
    def __call__(self, wave):
        """
        Compute Fitzpatrick99 extinction.
        
        Parameters
        ----------
        wave : float or `numpy.ndarray`
            Observed-frame wavelengths.  If no `unit` attribute available, 
            assume units are `astropy.units.Angstrom`.
        
        Returns
        -------
        Alambda : like ``wave``
            F99 extinction (mags) as a function of wavelength.  Output will
            be set to zero below 909 Angstroms and above 6 microns as the
            extinction modules themselves don't compute outside that range.
            
        """
        import astropy.units as u
        if not hasattr(wave, 'unit'):
            unit = u.Angstrom
        else:
            unit = 1
                       
        inwave = np.squeeze(wave).flatten()
        if self.module == 'dust_extinction.parameter_averages':
            clip = (inwave*unit > 1/10.*u.micron) 
            clip &= (inwave*unit < 1/0.3*u.micron)
        else:
            clip = (inwave*unit > 909*u.angstrom) & (inwave*unit < 6*u.micron)
        Alambda = np.zeros(inwave.shape)
                
        if clip.sum() == 0:
            return Alambda
        else:
            if self.module == 'dust_extinction.parameter_averages':
                flam = self.f99.extinguish(inwave[clip]*unit, Av=self.Av)
                Alambda[clip] = -2.5*np.log10(flam)
                
            elif self.module == 'specutils.extinction':
                Alambda[clip] = self.f99(inwave[clip]*unit)
            else:
                Alambda[clip] = self.f99(inwave[clip]*unit, self.Av)
        
        return Alambda

def abs_mag_to_luminosity(absmag, pivot=None, output_unit=u.L_sun):
    """
    Convert absolute AB mag to luminosity units
    
    Parameters
    ----------
    absmag : array-like
        Absolute AB magnitude.
    
    pivot : float
        Filter pivot wavelength associated with the magnitude.  If no units, 
        then assume `astropy.units.Angstrom`.
    
    output_unit : `astropy.units.core.Unit`
        Desired output unit.  Must specify a ``pivot`` wavelength for output
        power units, e.g., `astropy.unit.L_sun`.
    
    """
    if pivot is None:
        nu = 1.
    else:
        if hasattr(pivot, 'unit'):
            wunit = 1
        else:
            wunit = u.Angstrom
            
        nu = ((CLIGHT*u.m/u.second)/(pivot*wunit)).to(u.Hz)
        
    fjy = 3631*u.jansky * 10**(-0.4*absmag)
    d10 = (10*u.pc).to(u.cm)
    f10 = fjy * 4 * np.pi * d10**2 * nu
    return f10.to(output_unit)
    
def zphot_zspec(zphot, zspec, zlimits=None, zmin=0, zmax=4, axes=None, figsize=[6,7], minor=0.5, skip=2, selection=None, catastrophic_limit=0.15, title=None, min_zphot=0.02, alpha=0.2, extra_xlabel='', extra_ylabel='', xlabel=r'$z_\mathrm{spec}$', ylabel=r'$z_\mathrm{phot}$', label_pos=(0.05, 0.95), label_kwargs=dict(ha='left', va='top', fontsize=10), label_prefix='', format_axes=True, color='k', point_label=None, **kwargs):
    """
    Make zphot_zspec plot scaled by log(1+z) and show uncertainties

    Parameters
    ----------
    zphot : array-like
        Redshift on dependent axis

    zspec : array-like
        Redshift on independent axis

    zlimits : (N, 2) array
        Redshifts to use for photo-z errorbars, e.g. from 
        `~eazy.photoz.Photoz.pz_percentiles`, where `N` is the number of 
        objects as in `zphot` and `zspec`

    zmin, zmax : float
        Plot limits

    axes : `matplotlib` axes, None
        If specified, overplot in existing axes rather than generating a new
        plot.  For example, run the function once to generate the figure and
        then plot different points onto the existing axes:
        
        >>> fig = eazy.utils.zphot_spec(zphot, zspec, selection=sample1)
        >>> _ = eazy.utils.zphot_spec(zphot, zspec, selection=sample2, 
        >>>                           axes=fig.axes, color='b')

    figsize : list
        Figure canvas dimensions

    minor : float
        Axis tick interval

    skip : int
        Put axis labels every `skip` ticks

    selection : array-like
        Subsample selection (boolean or indices) applied as `zphot[selection]`

    catastrophic_limit : float
        Limit to define "catastrophic" failures, which is used for computing
        precision / outlier statistics printed on the plot

    title : str
        Title to add to the plot axes

    Returns
    -------
    fig : `matplotlib.figure.Figure`
        Figure object

    """
    import matplotlib.pyplot as plt
    from matplotlib.gridspec import GridSpec

    clip = (zphot > min_zphot) & (zspec > zmin) & (zspec <= zmax)

    if selection is not None:
        clip &= selection

    dz = (zphot-zspec)/(1+zspec)

    #izbest = np.argmin(self.fit_chi2, axis=1)

    clip_cat = (np.abs(dz) < catastrophic_limit)
    frac_cat = 1-(clip & clip_cat).sum() / clip.sum()
    NOUT = (clip & ~clip_cat).sum()
    
    gs = GridSpec(2,1, height_ratios=[6,1])
    NEW_AXES = axes is None
    if NEW_AXES:
        fig = plt.figure(figsize=figsize)
        ax = fig.add_subplot(gs[0,0])
    else:
        ax = axes[0]
        fig = None
        
    if title is not None:
        ax.set_title(title)

    if zlimits is not None:
        yerr = np.log10(1+np.abs(zlimits.T - zphot))
        ax.errorbar(np.log10(1+zspec[clip & ~clip_cat]), 
                    np.log10(1+zphot[clip & ~clip_cat]), 
                    yerr=yerr[:,clip & ~clip_cat], marker='.', alpha=alpha, 
                    color='r', linestyle='None')
        
        ax.errorbar(np.log10(1+zspec[clip & clip_cat]), 
                    np.log10(1+zphot[clip & clip_cat]), 
                    yerr=yerr[:,clip & clip_cat], marker='.', alpha=alpha, 
                    color=color, linestyle='None', label=point_label)
    else:
        ax.scatter(np.log10(1+zspec[clip & ~clip_cat]),
                   np.log10(1+zphot[clip & ~clip_cat]), 
                   marker='.', alpha=alpha, color='r')

        ax.scatter(np.log10(1+zspec[clip & clip_cat]), 
                   np.log10(1+zphot[clip & clip_cat]), 
                   marker='.', alpha=alpha, color=color, label=point_label)
        
    if NEW_AXES | format_axes:
        xt = np.arange(zmin, zmax+0.1, minor)
        xl = np.log10(1+xt)
        ax.plot(xl, xl, color='r', alpha=0.5)
        ax.set_xlim(xl[0], xl[-1])
        ax.set_ylim(xl[0],xl[-1])
        xtl = list(xt)

        if skip > 0:
            for i in range(1, len(xt), skip):
                xtl[i] = ''

        ax.set_xticks(xl)
        if axes is None:
            ax.set_xticklabels([])
        else:
            if len(axes) == 1:
                ax.set_xticks(xl)
                ax.set_xticklabels(xtl);
                ax.set_xlabel(xlabel + extra_xlabel)
            
        ax.set_yticks(xl); ax.set_yticklabels(xtl);
        ax.set_ylabel(ylabel + extra_ylabel)

    sample_nmad = nmad(dz[clip])
    sample_cat_nmad = nmad(dz[clip & clip_cat])

    if label_pos is not None:
        msg = r'{label_prefix} N={N} ({NOUT}, {err_frac:4.1f}%), $\sigma$={sample_nmad:.4f} ({sample_cat_nmad:.4f})'
        msg = msg.format(label_prefix=label_prefix, 
                         N=clip.sum(), err_frac=frac_cat*100, 
                         sample_nmad=sample_nmad, 
                         sample_cat_nmad=sample_cat_nmad, NOUT=NOUT)
                         
        ax.text(label_pos[0], label_pos[1], msg, transform=ax.transAxes)
    

    if axes is None:
        ax = fig.add_subplot(gs[1,0])
    else:
        if len(axes) == 2:
            ax = axes[1]
        else:
            return True
            
    if zlimits is not None:
        yerr = np.abs(zlimits.T-zphot)
        ax.errorbar(np.log10(1+zspec[clip & ~clip_cat]), dz[clip & ~clip_cat], 
                    yerr=yerr[:,clip & ~clip_cat], 
                    marker='.', alpha=alpha, color='r', linestyle='None')
                    
        ax.errorbar(np.log10(1+zspec[clip & clip_cat]), dz[clip & clip_cat], 
                    yerr=yerr[:,clip & clip_cat], 
                    marker='.', alpha=alpha, color='k', linestyle='None')
    else:
        ax.scatter(np.log10(1+zspec[clip & ~clip_cat]), dz[clip & ~clip_cat], 
                    marker='.', alpha=alpha, color='r')
        ax.scatter(np.log10(1+zspec[clip & clip_cat]), dz[clip & clip_cat],
                    marker='.', alpha=alpha, color='k')
        
    if fig is not None:
        ax.set_xticks(xl); ax.set_xticklabels(xtl);
        ax.set_xlim(xl[0], xl[-1])
        ax.set_ylim(-6*sample_nmad, 6*sample_nmad)
        ax.set_yticks([-3*sample_nmad, 0, 3*sample_nmad])
        ax.set_yticklabels([r'$-3\sigma$',r'$0$',r'$+3\sigma$'])
        ax.set_xlabel(xlabel + extra_xlabel)
        ax.set_ylabel(r'$\Delta z / 1+z$')
        for a in fig.axes:
            a.grid()

        fig.tight_layout(pad=0.1)
        return fig
    else:
        return True


def query_html(ra, dec, with_coords=True, replace_comma=True, queries=['CDS','ESO','MAST','ALMA', 'LEG','HSC'], **kwargs):
    """
    Return HTML string of queries around a position
    
    Parameters
    ----------
    ra, dec : float
        Coordinates in decimal degrees
    
    with_coords : bool
        Include '(ra, dec)' in output string
    
    replace_comma : bool
        Replace ',' with URL-safe '%2C'
    
    queries : list
        - CDS: Vizier/CDS catalogs
        - ESO: ESO archive
        - MAST: STScI/MAST HST archive
        - ALMA: ALMA archive
        - LEG/LEGACY: LegacySurvey map interface
        - HSC: HSC map interface
    
    Returns
    -------
    html : str
        HTML-formatted string with query links
        
    """
    if with_coords:
        html = [f"({ra:.6f}, {dec:.6f})"]
    else:
        html = []
    
    # Function/name mapping
    funcs = [cds_query, eso_query, mast_query, alma_query, show_legacysurvey, hscmap_query]
    names = ['CDS','ESO','MAST','ALMA', 'LEG','HSC']
    query_map = {}
    for name, func in zip(names, funcs):
        query_map[name] = func
    
    query_map['LEGACY'] = query_map['LEG']
    
    for name in queries:
        if name in query_map:
            func = query_map[name]
        else:
            continue
            
        url = func(ra, dec, **kwargs)
        html.append(f'<a href="{url}">{name}</a>')
    
    html = ' '.join(html)
    
    if replace_comma:
        html = html.replace(',','%2C')
        
    return html


def cds_query(ra, dec, radius=1., unit='s', **kwargs):
    """
    Open browswer with CDS catalog query around central position
    
    """
    #rd = self.get('pan fk5').strip()
    rd = f'{ra} {dec}'
    rdst = rd.replace('+', '%2B').replace('-', '%2D').replace(' ', '+')
    url = (f'http://vizier.u-strasbg.fr/viz-bin/VizieR?'
           f'-c={rdst}&-c.r{unit}={radius}')
           
    #os.system(f'open {url}')
    return url


def eso_query(ra, dec, radius=1., unit='m', dp_types=['CUBE','IMAGE'], extra='', **kwargs):
    """
    Open browser with ESO archive query around central position.
    
    Note: ESO query is data footprint **contains*** point
    
    """
    #ra, dec = self.get('pan fk5').strip().split()
    
    # native is deg
    if unit == 'd':
        r = f'{radius:.2f}'
    elif unit == 'm':
        r = f'{radius/60:.3f}'
    elif unit == 's':
        r = f'{radius/3600:.5f}'
        
    dp_type = ','.join(dp_types)
    
    url = (f'https://archive.eso.org/scienceportal/home?'
            f'pos={ra},{dec}&r={r}&dp_type={dp_type}{extra}')
                    
    #os.system(f'open {url}')
    return url


def mast_query(ra, dec, instruments=['WFC3','ACS','WFPC2'], mast_radius=1., mast_unit='m', max=1000, **kwargs):
    """
    Open browser with MAST archive query around central position
    
    Note: MAST query is **distance to** point
    
    """
    #ra, dec = self.get('pan fk5').strip().split()
    if len(instruments) > 0:
        instr='&sci_instrume='+','.join(instruments)
    else:
        instr = ''
    
    # native is arcmin
    if mast_unit == 'd':
        r = f'{mast_radius*60:.2f}'
    elif mast_unit == 'm':
        r = f'{mast_radius:.3f}'
    elif mast_unit == 's':
        r = f'{mast_radius/60:.5f}'
        
    url = (f'https://archive.stsci.edu/hst/search.php?RA={ra}&DEC={dec}'
           f'&radius={r}'
           f'&sci_aec=S{instr}&max_records={max}&outputformat=HTML_Table'
            '&action=Search')
            
    #os.system(f'open {url}')
    return url


def alma_query(ra, dec, mirror="almascience.eso.org", radius=1, unit='m', extra='', **kwargs):
    """
    Open browser with ALMA archive query around central position
    """

    # native is arcmin
    if unit == 'd':
        r = f'{radius*60:.2f}'
    elif unit == 'm':
        r = f'{radius:.3f}'
    elif unit == 's':
        r = f'{radius/60:.5f}'
        
    url = (f"https://{mirror}/aq/?result_view=observation"
           f"&raDec={ra}%20{dec},{r}{extra}")
    #os.system(f'open "{url}"')
    return url


def hscmap_query(ra, dec, open=True, **kwargs):
    """
    Function to open HSC explorer in browser centered on target coordinates
    """
    
    import os
    rrad = ra/180*np.pi
    drad = dec/180*np.pi
    url = (f"https://hsc-release.mtk.nao.ac.jp/hscMap-pdr2/app/#/?_=%7B%22view%22%3A%7B%22a%22%3A{rrad},%22d%22%3A{drad}"
           ",%22fovy%22%3A0.00009647627785850188,%22roll%22%3A0%7D,%22sspParams%22%3A%7B%22type%22%3A%22"
           "SDSS_TRUE_COLOR%22,%22filter%22%3A%5B%22HSC-Y%22,%22HSC-Z%22,%22HSC-I%22%5D,%22simpleRgb"
           "%22%3A%7B%22beta%22%3A22026.465794806718,%22a%22%3A1,%22bias%22%3A0.05,%22b0%22%3A0%7D,%22"
           "sdssTrueColor%22%3A%7B%22beta%22%3A40106.59228119989,%22a%22%3A2.594451857120983,%22bias%22%3A0.05,"
           "%22b0%22%3A0%7D%7D,%22externalTiles%22%3A%5B%5D,%22activeReruns%22%3A%5B%22pdr2_wide%22,%22pdr2_dud"
           "%22%5D%7D")
        
    return url


def show_legacysurvey(ra, dec, layer='dr8', zoom=17, **kwargs):
    """
    Open browser with legacysurvey.org panner around central position
    """
    #ra, dec = self.get('pan fk5').strip().split()
    url = (f'http://legacysurvey.org/viewer?ra={ra}&dec={dec}'
           f'&layer={layer}&zoom={zoom}')
            
    #os.system(f'open {url}')
    return url


def interp_conserve(x, xp, fp, left=0., right=0.):
    """
    Interpolation analogous to `numpy.interp` but conserving "flux".
    
    Parameters
    ----------
    x : `numpy.ndarray`
        Desired interpolation locations

    xp, fp : `numpy.ndarray`
        The `x` and `y` coordinates of the function to be interpolated.  The
        `x` array can be irregularly spaced but should be increase
        monotonically.
    
    left, right : float
        Values to use for extrapolation below the minimum and maximum limits
        of `x`.
        
    Returns
    -------
    y : like `x`
        Interpolated values.
    
    Interpolation performed by trapezoidal integration between the midpoints
    of the output `x` array with `numpy.trapz`.
    
    .. note:: For a faster `cython` implementation of this function, see 
              `grizli.utils_c.interp_conserve_c`.
              
    """
    midpoint = (x[1:]-x[:-1])/2.+x[:-1]
    midpoint = np.append(midpoint, np.array([x[0],x[-1]]))
    midpoint = midpoint[np.argsort(midpoint)]
    int_midpoint = np.interp(midpoint, xp, fp, left=left, right=right)
    int_midpoint[midpoint > xp.max()] = right
    int_midpoint[midpoint < xp.min()] = left
    
    fullx = np.append(xp, midpoint)
    fully = np.append(fp, int_midpoint)
    
    so = np.argsort(fullx)
    fullx, fully = fullx[so], fully[so]
    
    outy = x*0.
    dx = midpoint[1:]-midpoint[:-1]
    for i in range(len(x)):
        bin = (fullx >= midpoint[i]) & (fullx <= midpoint[i+1])
        outy[i] = np.trapz(fully[bin], fullx[bin])/dx[i]
        
    return outy


class emceeChain():     
    def __init__(self, chain=None, file=None, param_names=[],
                       burn_fraction=0.5, sampler=None):
        
        self.param_names = []
        
        if chain is not None:
            self.chain = chain
        
        if file is not None:
            if 'fits' in file.lower():
                self.load_fits(file=file)            
            else:
                self.load_chain(file=file)
                     
        self.process_chain(param_names = param_names,
                           burn_fraction=burn_fraction)
        #
        if sampler is not None:
            from numpy import unravel_index
            max_ix = unravel_index(sampler.lnprobability.argmax(), sampler.lnprobability.shape)
            self.map = self.chain[max_ix[0], max_ix[1],:]
            self.is_map = True
        else:
            self.map = self.median
            self.is_map = False
            
    def process_chain(self, param_names=[], burn_fraction=0.5):
        """
        Define parameter names and get parameter statistics
        """
        self.nwalkers, self.nstep, self.nparam = self.chain.shape
                
        if param_names == []:            
            if self.param_names == []:
                for i in range(self.nparam):
                    param_names.append('a%d' %(i+1))
               
                self.param_names = param_names
        
        else:
            if len(param_names) != self.nparam:
                print('param_names must have N=%d (or zero) entries' %(self.nparam))
                return False
                        
            self.param_names = param_names
                
        self.param_dict = {}
        for i in range(self.nparam):
            self.param_dict[self.param_names[i]] = i
        
        self.nburn = int(np.round(burn_fraction*self.nstep))
        self.stats = {}
        self.median = np.zeros(self.nparam)
                
        for param in self.param_names:
            pid = self.param_dict[param]
            self.stats[param] = self.get_stats(pid, burn=self.nburn)
            self.median[pid] = self.stats[param]['q50']
            
    def get_stats(self, pid, burn=0, raw=False):
        """
        Get percentile statistics for a parameter in the chain
        """
        if raw:
            pchain = pid*1.
        else:
            pchain = self.chain[:,burn:,pid].flatten()
        
        stats = {}
        stats['q05'] = np.percentile(pchain, 5)
        stats['q16'] = np.percentile(pchain, 16)
        stats['q50'] = np.percentile(pchain, 50)
        stats['q84'] = np.percentile(pchain, 84)
        stats['q95'] = np.percentile(pchain, 95)
        stats['mean'] = np.mean(pchain)
        stats['std'] = np.std(pchain)
        stats['width'] = (stats['q84']-stats['q16'])/2.
        return stats
        
    def show_chain(self, param='a1', chain=None, alpha=0.15, color='blue', scale=1, diff=0, ax = None, add_labels=True, hist=False, autoscale=True, *args, **kwargs):
        """
        Make a plot of the chain for a given parameter.
        
        For plotting, multiply the parameter by `scale` and subtract `diff`.
        
        """
        if chain is None:
            pid = self.param_dict[param]
            chain = self.chain[:,:,pid]
        
        if ax is not None:
            plotter = ax
            xlabel = ax.set_xlabel
            ylabel = ax.set_ylabel
            ylim = ax.set_ylim
        else:
            plotter = plt
            xlabel = plt.xlabel
            ylabel = plt.ylabel
            ylim = plt.ylim
            
        if hist:
            h = plotter.hist(chain[:,self.nburn:].flatten(), alpha=alpha, color=color, *args, **kwargs)
            if add_labels:
                ylabel('N')
                xlabel(param)
        else:
            for i in range(self.nwalkers):
                p = plotter.plot(chain[i,:]*scale-diff, alpha=alpha, color=color, *args, **kwargs)
            if add_labels:
                xlabel('Step')
                ylabel(param)
            #
            if autoscale:
                ylim(self.stats[param]['q50']*scale + np.array([-8,8])*self.stats[param]['width']*scale)
            
    def save_chain(self, file='emcee_chain.pkl', verbose=True):
        """
        Save the chain to a Pkl file
        """
        import cPickle as pickle
        
        fp = open(file,'wb')
        pickle.dump(self.nwalkers, fp)
        pickle.dump(self.nstep, fp)
        pickle.dump(self.nparam, fp)
        pickle.dump(self.param_names, fp)
        pickle.dump(self.chain, fp)
        fp.close()
        
        if verbose:
            print('Wrote %s.' %(file))
        
    def load_chain(self, file='emcee_chain.pkl'):
        """
        Read the chain from the pickle file
        """
        import cPickle as pickle
        
        fp = open(file, 'rb')
        self.nwalkers = pickle.load(fp)
        self.nstep = pickle.load(fp)
        self.nparam = pickle.load(fp)
        self.param_names = pickle.load(fp)
        self.chain = pickle.load(fp)
        fp.close()
    
    def save_fits(self, file='emcee_chain.fits', verbose=True):
        """
        Make a FITS file of an EMCEE chain
        """
        import astropy.io.fits as pyfits
        
        header = pyfits.Header()
        header.update('NWALKERS', self.nwalkers)
        header.update('NSTEP', self.nstep)
        header.update('NPARAM', self.nparam)
        
        hdu = [pyfits.PrimaryHDU(header=header)]
        
        for param in self.param_names:
            header.update('PARAM', param)
            hdu.append(pyfits.ImageHDU(data=self.__getitem__(param), header=header, name=param))
        
        hduList = pyfits.HDUList(hdu)
        hduList.writeto(file, clobber=True, output_verify='silentfix')
    
        if verbose:
            print('Wrote %s.' %(file))
    
    def load_fits(self, file='emcee_chain.fits'):
        """
        Load emcee chain fits file created by ``save_fits``.
        """
        import astropy.io.fits as pyfits
        
        im = pyfits.open(file)
        self.nwalkers = im[0].header['NWALKERS']
        self.nstep = im[0].header['NSTEP']
        self.nparam = im[0].header['NPARAM']
        self.param_names = []
        self.chain = np.ones((self.nwalkers, self.nstep, self.nparam))
        for i in range(self.nparam):
            self.param_names.append(im[i+1].header['PARAM'])
            self.chain[:,:,i] = im[i+1].data
        
        im.close()
    
    def parameter_correlations(self, size=8, shrink=5, show=None, file=None):
        if show is None:
            show = self.param_names
        
        NP = len(show)
        #fig = unicorn.plotting.plot_init(square=True, aspect=1, xs=size, left=0.05, right=0.01, bottom=0.01, top=0.01, NO_GUI=False, use_tex=False, fontsize=7)
        fig = plt.figure(figsize=(7,7))
        fig.subplots_adjust(wspace=0.0,hspace=0.0)
        
        counter = 0
        for i in range(NP):
            for j in range(NP):
                counter = counter + 1
                ax = fig.add_subplot(NP, NP, counter)
                a = ax.plot(self[show[i]][:,self.nburn::shrink].flatten(), self[show[j]][:,self.nburn::shrink].flatten(), alpha=0.03, color='black', linestyle='None', marker=',')
                a = ax.set_xlim(self.stats[show[i]]['q50']-3*self.stats[show[i]]['std'], self.stats[show[i]]['q50']+3*self.stats[show[i]]['std'])
                a = ax.set_ylim(self.stats[show[j]]['q50']-3*self.stats[show[j]]['std'], self.stats[show[j]]['q50']+3*self.stats[show[j]]['std'])
                if i == j:
                    a = ax.text(0.5, 0.92, show[i], fontsize=8, color='red', horizontalalignment='center', verticalalignment='top', transform=ax.transAxes)

        if file is not None:
            fig.savefig(file)
    
    def draw_random(self, N=10):
        """
        Draw random sets of parameters from the chain
        """
        #ok_walk = self.sampler.acceptance_fraction > min_acceptance
        iwalk = np.asarray(np.random.rand(N)*self.nwalkers,dtype=int)
        istep = self.nburn + np.asarray(np.random.rand(N)*(self.nstep-self.nburn),dtype=int)
        draw = self.chain[iwalk, istep, :]
        return draw
        
    def __getitem__(self, param):
        pid = self.param_dict[param]
        return self.chain[:,:,pid]
        
    def contour(self, p1, p2, labels=None, levels=[0.683, 0.955], colors=None, limits=None, bins=20, ax=None, fill=False, **kwargs):
        """
        Plot sigma contours
        """
        import astroML.plotting
        
        if isinstance(p1, str):
            trace1 = self.__getitem__(p1)[:,self.nburn:].flatten()
            pname1 = p1
        else:
            trace1 = p1.flatten()
            pname1 = ''
        
        if isinstance(p2, str):
            trace2 = self.__getitem__(p2)[:,self.nburn:].flatten()
            pname2 = p2
        else:
            trace2 = p2.flatten()
            pname2 = ''
        
        if labels is None:
            labels = [pname1, pname2]
            
        if limits is None:
            limits = [(t.min(), t.max()) for t in [trace1, trace2]]
        
        bins = [np.linspace(limits[i][0], limits[i][1], bins + 1)
                for i in range(2)]
        
        
        H, xbins, ybins = np.histogram2d(trace1, trace2, bins=(bins[0], bins[1]))
        H[H == 0] = 1E-16
        Nsigma = astroML.plotting.mcmc.convert_to_stdev(np.log(H))
        
        if ax is None:
            ax = plt
        
        ax.contour(0.5 * (xbins[1:] + xbins[:-1]),
                   0.5 * (ybins[1:] + ybins[:-1]),
                   Nsigma.T, levels=levels, **kwargs)
        
        if fill:
            if ax is plt:
                col = plt.gca().collections
            else:
                col = ax.collections
            
            n_levels = len(levels)
            if colors is None:
                dc = 1./(n_levels+1)
                colors = ['%.2f' %((i+1)*dc) for i in np.arange(n_levels)]
                print(colors)

            for i in range(n_levels): 
                print(colors[i])
                col[i].set_facecolor(colors[i])
                col[i].set_edgecolor(colors[i])
                col[i].set_zorder(-10-i)
                col[i].set_alpha(0.8)
                
                
        if ax is plt:
            ax.xlabel(labels[0])
            ax.ylabel(labels[1])
            ax.xlim(limits[0])
            ax.ylim(limits[1])
        else:
            ax.set_xlabel(labels[0])
            ax.set_ylabel(labels[1])
            ax.set_xlim(limits[0])
            ax.set_ylim(limits[1])



import os
import numpy as np
#from math import pow as _pow

from . import __file__ as filepath

__all__ = ["Asada24","Inoue14"]

class Asada24(object):
    def __init__(self, sigmoid_params=(3.48347968, 1.25809685, 18.24922789), scale_tau=1., add_cgm=True):
        """
        Compute IGM+CGM transmission from Asada et al. 2024, in prep.
        The IGM model is from Inoue+ (2014).
        
        Parameters
        ----------
        sigmoid_params : 3-tuple of float
            Parameters that controll the redshift evolution of the CGM HI gas column density.
            The defaul values are from Asada et al. (2024).
        
        scale_tau : float
            Scalar multiplied to tau_igm
            
        add_cgm : bool
            Add the additional LyA damping absorption at z>6 as described in Asada+24.
            If False, the transmission will be identical to Inoue+ 2014
            
        .. plot::
            :include-source:
            
            # Compare two IGM transmissions
            
            import numpy as np
            import matplotlib.pyplot as plt
            from eazy import igm as igm_module
            
            igm_A24 = igm_module.Asada24()
            igm_I14 = igm_module.Inoue14()

            redshifts = [6., 7., 8., 9., 10.]
            colors = ['b', 'c', 'purple', 'orange', 'red']
            
            wave = np.linspace(100,2000,1901) ## wavelength array in the rest-frame
            lyman = wave < 2000


            fig = plt.figure(figsize=(6,5))
            for z, c in zip(redshifts, colors):
                igmz_A24 = wave*0.+1
                igmz_A24[lyman] = igm_A24.full_IGM(z, (wave*(1+z))[lyman])

                igmz_I14 = wave*0.+1
                igmz_I14[lyman] = igm_I14.full_IGM(z, (wave*(1+z))[lyman])

                plt.plot(wave*(1+z), igmz_I14, color=c, ls='dashed')
                plt.plot(wave*(1+z), igmz_A24, color=c, label=r'$z={}$'.format(int(z)))
                
            plt.xlabel('Observed wavelength [A]')
            plt.ylabel('Transmission')

            plt.legend()

            plt.xlim(5000,17000)
        """
        self._load_data()
        self.sigmoid_params = sigmoid_params
        self.scale_tau = scale_tau
        self.add_cgm = add_cgm
        

    def tau_cgm(self, N_HI, lam, z):
        """
        CGM optical depth given by Totani+06, eqn (1)

        Parameters
        ----------
        N_HI : float
            HI column density [cm-2]

        lam : 1D array
            wavelength array in the observed frame [AA]

        z : float
            Redshift of the source

        Returns
        -------
        1D array of tau_CGM
        """
        
        lam_rest = lam/(1+z)
        nu_rest = 3e18/lam_rest

        tau = np.zeros(len(lam))

        for i, nu in enumerate(nu_rest):
            tau[i] = N_HI * sigma_a(nu)

        return tau
        
    def lgNHI_z(self, z):
        """
        HI column density as a function of redshift, calibrated in Asada+ 2024.
        Only valid at z>=6
        
        Parameters
        ----------
        z : float
            Redshift of the source

        Returns
        -------
        log10(HI column density [cm-2])
        """
        
        lgN_HI = sigmoid(z, self.sigmoid_params[0], self.sigmoid_params[1], self.sigmoid_params[2])
        
        return lgN_HI
        
    ### IGM part -- identical to Inoue+14
    def _load_data(self):
        path = os.path.join(os.path.dirname(filepath),'data')
        #print path
    
        LAF_file = os.path.join(path, 'LAFcoeff.txt')
        DLA_file = os.path.join(path, 'DLAcoeff.txt')
    
        data = np.loadtxt(LAF_file, unpack=True)
        ix, lam, ALAF1, ALAF2, ALAF3 = data
        self.lam = lam[:,np.newaxis]
        self.ALAF1 = ALAF1[:,np.newaxis]
        self.ALAF2 = ALAF2[:,np.newaxis]
        self.ALAF3 = ALAF3[:,np.newaxis]
        
        data = np.loadtxt(DLA_file, unpack=True)
        ix, lam, ADLA1, ADLA2 = data
        self.ADLA1 = ADLA1[:,np.newaxis]
        self.ADLA2 = ADLA2[:,np.newaxis]
                
        return True


    @property
    def NA(self):
        """
        Number of Lyman-series lines
        """
        return self.lam.shape[0]


    def tLSLAF(self, zS, lobs):
        """
        Lyman series, Lyman-alpha forest
        """
        z1LAF = 1.2
        z2LAF = 4.7

        l2 = self.lam #[:, np.newaxis]
        tLSLAF_value = np.zeros_like(lobs*l2).T
        
        x0 = (lobs < l2*(1+zS))
        x1 = x0 & (lobs < l2*(1+z1LAF))
        x2 = x0 & ((lobs >= l2*(1+z1LAF)) & (lobs < l2*(1+z2LAF)))
        x3 = x0 & (lobs >= l2*(1+z2LAF))
        
        tLSLAF_value = np.zeros_like(lobs*l2)
        tLSLAF_value[x1] += ((self.ALAF1/l2**1.2)*lobs**1.2)[x1]
        tLSLAF_value[x2] += ((self.ALAF2/l2**3.7)*lobs**3.7)[x2]
        tLSLAF_value[x3] += ((self.ALAF3/l2**5.5)*lobs**5.5)[x3]

        return tLSLAF_value.sum(axis=0)


    def tLSDLA(self, zS, lobs):
        """
        Lyman Series, DLA
        """
        z1DLA = 2.0
        
        l2 = self.lam #[:, np.newaxis]
        tLSDLA_value = np.zeros_like(lobs*l2)
        
        x0 = (lobs < l2*(1+zS)) & (lobs < l2*(1.+z1DLA))
        x1 = (lobs < l2*(1+zS)) & ~(lobs < l2*(1.+z1DLA))
        
        tLSDLA_value[x0] += ((self.ADLA1/l2**2)*lobs**2)[x0]
        tLSDLA_value[x1] += ((self.ADLA2/l2**3)*lobs**3)[x1]
                
        return tLSDLA_value.sum(axis=0)


    def tLCDLA(self, zS, lobs):
        """
        Lyman continuum, DLA
        """
        z1DLA = 2.0
        lamL = 911.8
        
        tLCDLA_value = np.zeros_like(lobs)
        
        x0 = lobs < lamL*(1.+zS)
        if zS < z1DLA:
            tLCDLA_value[x0] = 0.2113 * (1.0+zS)**2.0 - 0.07661 * (1.0+zS)**2.3 * (lobs[x0]/lamL)**(-3e-1) - 0.1347 * (lobs[x0]/lamL)**2.0
        else:
            x1 = lobs >= lamL*(1.+z1DLA)
            
            tLCDLA_value[x0 & x1] = 0.04696 * (1.0+zS)**3.0 - 0.01779 * (1.0+zS)**3.3 * (lobs[x0 & x1]/lamL)**(-3e-1) - 0.02916 * (lobs[x0 & x1]/lamL)**3.0
            tLCDLA_value[x0 & ~x1] =0.6340 + 0.04696 * (1.0+zS)**3.0 - 0.01779 * (1.0+zS)**3.3 * (lobs[x0 & ~x1]/lamL)**(-3e-1) - 0.1347 * (lobs[x0 & ~x1]/lamL)**2.0 - 0.2905 * (lobs[x0 & ~x1]/lamL)**(-3e-1)
        
        return tLCDLA_value


    def tLCLAF(self, zS, lobs):
        """
        Lyman continuum, LAF
        """
        z1LAF = 1.2
        z2LAF = 4.7
        lamL = 911.8

        tLCLAF_value = np.zeros_like(lobs)
        
        x0 = lobs < lamL*(1.+zS)
        
        if zS < z1LAF:
            tLCLAF_value[x0] = 0.3248 * ( (lobs[x0]/lamL)**1.2 - (1.0+zS)**(-9e-1) * (lobs[x0]/lamL)**2.1 )
        elif zS < z2LAF:
            x1 = lobs >= lamL*(1+z1LAF)
            tLCLAF_value[x0 & x1] = 2.545e-2 * ( (1.0+zS)**1.6 * (lobs[x0 & x1]/lamL)**2.1 - (lobs[x0 & x1]/lamL)**3.7 )
            tLCLAF_value[x0 & ~x1] = 2.545e-2 * (1.0+zS)**1.6 * (lobs[x0 & ~x1]/lamL)**2.1 + 0.3248 * (lobs[x0 & ~x1]/lamL)**1.2 - 0.2496 * (lobs[x0 & ~x1]/lamL)**2.1
        else:
            x1 = lobs > lamL*(1.+z2LAF)
            x2 = (lobs >= lamL*(1.+z1LAF)) & (lobs < lamL*(1.+z2LAF))
            x3 = lobs < lamL*(1.+z1LAF)
            
            tLCLAF_value[x0 & x1] = 5.221e-4 * ( (1.0+zS)**3.4 * (lobs[x0 & x1]/lamL)**2.1 - (lobs[x0 & x1]/lamL)**5.5 )
            tLCLAF_value[x0 & x2] = 5.221e-4 * (1.0+zS)**3.4 * (lobs[x0 & x2]/lamL)**2.1 + 0.2182 * (lobs[x0 & x2]/lamL)**2.1 - 2.545e-2 * (lobs[x0 & x2]/lamL)**3.7
            tLCLAF_value[x0 & x3] = 5.221e-4 * (1.0+zS)**3.4 * (lobs[x0 & x3]/lamL)**2.1 + 0.3248 * (lobs[x0 & x3]/lamL)**1.2 - 3.140e-2 * (lobs[x0 & x3]/lamL)**2.1
            
        return tLCLAF_value
        
    def full_IGM(self, z, lobs):
        """Get full IGM+CGM absorption
        
        Parameters
        ----------
        z : float
            Redshift to evaluate IGM absorption
        
        lobs : array-like
            Observed-frame wavelength(s) in Angstroms.
        
        Returns
        -------
        abs : array-like
            IGM+CGM transmission factor
        
        """
        
        tau_LS = self.tLSLAF(z, lobs) + self.tLSDLA(z, lobs)
        tau_LC = self.tLCLAF(z, lobs) + self.tLCDLA(z, lobs)
        tau_clip = 0.
        
        igmz = np.exp(-self.scale_tau*(tau_LC + tau_LS + tau_clip))
        
        if self.add_cgm:
            if (z<6):
                tau_C = np.zeros(len(lobs))
            else:
                NHI = 10**(self.lgNHI_z(z))
                tau_C = self.tau_cgm(NHI, lobs, z)
            cgmz = np.exp(-tau_C)
        else:
            cgmz = np.ones(len(lobs))
        
        return igmz * cgmz


class Inoue14(object):
    def __init__(self, scale_tau=1.):
        """
        IGM absorption from Inoue et al. (2014)
        
        Parameters
        ----------
        scale_tau : float
            Parameter multiplied to the IGM :math:`\tau` values (exponential
            in the linear absorption fraction).
            I.e., :math:`f_\mathrm{igm} = e^{-\mathrm{scale\_tau} \tau}`.
        """
        self._load_data()
        self.scale_tau = scale_tau


    def _load_data(self):
        path = os.path.join(os.path.dirname(filepath),'data')
        #print path
    
        LAF_file = os.path.join(path, 'LAFcoeff.txt')
        DLA_file = os.path.join(path, 'DLAcoeff.txt')
    
        data = np.loadtxt(LAF_file, unpack=True)
        ix, lam, ALAF1, ALAF2, ALAF3 = data
        self.lam = lam[:,np.newaxis]
        self.ALAF1 = ALAF1[:,np.newaxis]
        self.ALAF2 = ALAF2[:,np.newaxis]
        self.ALAF3 = ALAF3[:,np.newaxis]
        
        data = np.loadtxt(DLA_file, unpack=True)
        ix, lam, ADLA1, ADLA2 = data
        self.ADLA1 = ADLA1[:,np.newaxis]
        self.ADLA2 = ADLA2[:,np.newaxis]
                
        return True


    @property
    def NA(self):
        """
        Number of Lyman-series lines
        """
        return self.lam.shape[0]


    def tLSLAF(self, zS, lobs):
        """
        Lyman series, Lyman-alpha forest
        """
        z1LAF = 1.2
        z2LAF = 4.7

        l2 = self.lam #[:, np.newaxis]
        tLSLAF_value = np.zeros_like(lobs*l2).T
        
        x0 = (lobs < l2*(1+zS))
        x1 = x0 & (lobs < l2*(1+z1LAF))
        x2 = x0 & ((lobs >= l2*(1+z1LAF)) & (lobs < l2*(1+z2LAF)))
        x3 = x0 & (lobs >= l2*(1+z2LAF))
        
        tLSLAF_value = np.zeros_like(lobs*l2)
        tLSLAF_value[x1] += ((self.ALAF1/l2**1.2)*lobs**1.2)[x1]
        tLSLAF_value[x2] += ((self.ALAF2/l2**3.7)*lobs**3.7)[x2]
        tLSLAF_value[x3] += ((self.ALAF3/l2**5.5)*lobs**5.5)[x3]

        return tLSLAF_value.sum(axis=0)


    def tLSDLA(self, zS, lobs):
        """
        Lyman Series, DLA
        """
        z1DLA = 2.0
        
        l2 = self.lam #[:, np.newaxis]
        tLSDLA_value = np.zeros_like(lobs*l2)
        
        x0 = (lobs < l2*(1+zS)) & (lobs < l2*(1.+z1DLA))
        x1 = (lobs < l2*(1+zS)) & ~(lobs < l2*(1.+z1DLA))
        
        tLSDLA_value[x0] += ((self.ADLA1/l2**2)*lobs**2)[x0]
        tLSDLA_value[x1] += ((self.ADLA2/l2**3)*lobs**3)[x1]
                
        return tLSDLA_value.sum(axis=0)


    def tLCDLA(self, zS, lobs):
        """
        Lyman continuum, DLA
        """
        z1DLA = 2.0
        lamL = 911.8
        
        tLCDLA_value = np.zeros_like(lobs)
        
        x0 = lobs < lamL*(1.+zS)
        if zS < z1DLA:
            tLCDLA_value[x0] = 0.2113 * (1.0+zS)**2.0 - 0.07661 * (1.0+zS)**2.3 * (lobs[x0]/lamL)**(-3e-1) - 0.1347 * (lobs[x0]/lamL)**2.0
        else:
            x1 = lobs >= lamL*(1.+z1DLA)
            
            tLCDLA_value[x0 & x1] = 0.04696 * (1.0+zS)**3.0 - 0.01779 * (1.0+zS)**3.3 * (lobs[x0 & x1]/lamL)**(-3e-1) - 0.02916 * (lobs[x0 & x1]/lamL)**3.0
            tLCDLA_value[x0 & ~x1] =0.6340 + 0.04696 * (1.0+zS)**3.0 - 0.01779 * (1.0+zS)**3.3 * (lobs[x0 & ~x1]/lamL)**(-3e-1) - 0.1347 * (lobs[x0 & ~x1]/lamL)**2.0 - 0.2905 * (lobs[x0 & ~x1]/lamL)**(-3e-1)
        
        return tLCDLA_value


    def tLCLAF(self, zS, lobs):
        """
        Lyman continuum, LAF
        """
        z1LAF = 1.2
        z2LAF = 4.7
        lamL = 911.8

        tLCLAF_value = np.zeros_like(lobs)
        
        x0 = lobs < lamL*(1.+zS)
        
        if zS < z1LAF:
            tLCLAF_value[x0] = 0.3248 * ( (lobs[x0]/lamL)**1.2 - (1.0+zS)**(-9e-1) * (lobs[x0]/lamL)**2.1 )
        elif zS < z2LAF:
            x1 = lobs >= lamL*(1+z1LAF)
            tLCLAF_value[x0 & x1] = 2.545e-2 * ( (1.0+zS)**1.6 * (lobs[x0 & x1]/lamL)**2.1 - (lobs[x0 & x1]/lamL)**3.7 )
            tLCLAF_value[x0 & ~x1] = 2.545e-2 * (1.0+zS)**1.6 * (lobs[x0 & ~x1]/lamL)**2.1 + 0.3248 * (lobs[x0 & ~x1]/lamL)**1.2 - 0.2496 * (lobs[x0 & ~x1]/lamL)**2.1
        else:
            x1 = lobs > lamL*(1.+z2LAF)
            x2 = (lobs >= lamL*(1.+z1LAF)) & (lobs < lamL*(1.+z2LAF))
            x3 = lobs < lamL*(1.+z1LAF)
            
            tLCLAF_value[x0 & x1] = 5.221e-4 * ( (1.0+zS)**3.4 * (lobs[x0 & x1]/lamL)**2.1 - (lobs[x0 & x1]/lamL)**5.5 )
            tLCLAF_value[x0 & x2] = 5.221e-4 * (1.0+zS)**3.4 * (lobs[x0 & x2]/lamL)**2.1 + 0.2182 * (lobs[x0 & x2]/lamL)**2.1 - 2.545e-2 * (lobs[x0 & x2]/lamL)**3.7
            tLCLAF_value[x0 & x3] = 5.221e-4 * (1.0+zS)**3.4 * (lobs[x0 & x3]/lamL)**2.1 + 0.3248 * (lobs[x0 & x3]/lamL)**1.2 - 3.140e-2 * (lobs[x0 & x3]/lamL)**2.1
            
        return tLCLAF_value


    def full_IGM(self, z, lobs):
        """Get full Inoue IGM absorption
        
        Parameters
        ----------
        z : float
            Redshift to evaluate IGM absorption
        
        lobs : array
            Observed-frame wavelength(s) in Angstroms.
        
        Returns
        -------
        abs : array
            IGM absorption
        
        """
        tau_LS = self.tLSLAF(z, lobs) + self.tLSDLA(z, lobs)
        tau_LC = self.tLCLAF(z, lobs) + self.tLCDLA(z, lobs)
        
        ### Upturn at short wavelengths, low-z
        #k = 1./100
        #l0 = 600-6/k
        #clip = lobs/(1+z) < 600.
        #tau_clip = 100*(1-1./(1+np.exp(-k*(lobs/(1+z)-l0))))
        tau_clip = 0.
        
        return np.exp(-self.scale_tau*(tau_LC + tau_LS + tau_clip))


    def build_grid(self, zgrid, lrest):
        """Build a spline interpolation object for fast IGM models
        
        Returns: self.interpolate
        """
        
        from scipy.interpolate import CubicSpline
        igm_grid = np.zeros((len(zgrid), len(lrest)))
        for iz in range(len(zgrid)):
            igm_grid[iz,:] = self.full_IGM(zgrid[iz], lrest*(1+zgrid[iz]))
        
        self.interpolate = CubicSpline(zgrid, igm_grid)


    
def sigmoid(x,A,a,c):
    """
    Sigmoid function centered at x=6
    """

    return A/(1+np.exp(-a*(x-6))) + c
    
    
def sigma_a(nu_rest):
    """
     Lyα absorption cross section for the restframe frequency \nu given by Totani+06, eqn (1)
     for CGM damping wing calculation

    Parameters
    ----------
    nu : float
        rest-frame frequency [Hz]

    Returns
    -------
    Lyα absorption cross section at the restframe frequency \nu_rest [cm2]
    """
    
    Lam_a = 6.255486e8 ## Hz
    nu_lya = 2.46607e15 ## Hz

    C = 6.9029528e22 ## Constant factor [Ang2/s2]

    sig = C *(nu_rest/nu_lya)**4 / (4*np.pi**2 * (nu_rest - nu_lya)**2 +  Lam_a**2*(nu_rest/nu_lya)**6/4)

    s = sig * 1e-16 ## convert AA-2 to cm-2

    return s

    



import os
import warnings

import numpy as np

from astropy.utils.exceptions import AstropyWarning

from .. import filters, fetch_eazy_photoz

def test_array_filter():
    """
    Generate filter from arrays
    """
    wx = np.arange(5400, 5600., 1)
    wy = wx*0.
    wy[10:-10] = 1
    
    f1 = filters.FilterDefinition(wave=wx, throughput=wy, name='Tophat 5500')
    print('xxx', f1.pivot)
    assert(np.allclose(f1.pivot, 5500, rtol=1.e-3))
    assert(np.allclose(f1.ABVega, 0.016, atol=0.03))
    assert(np.allclose(f1.equivwidth, 180))
    assert(np.allclose(f1.rectwidth, 180))


def test_pysynphot_filter():
    """
    PySynphot filter bandpass
    """
    try:
        import pysynphot as S
    except:
        return True
    
    with warnings.catch_warnings():
        warnings.simplefilter('ignore', AstropyWarning)
        v_pysyn = S.ObsBandpass('v')
        
    v_eazy = filters.FilterDefinition(bp=v_pysyn)
    
    assert(np.allclose(v_pysyn.pivot(), v_eazy.pivot, rtol=0.001))


def test_data_path():
    """
    Data path
    """
    path = os.path.join(os.path.dirname(__file__), '../data/')
    assert(os.path.exists(path))
    
    if not os.path.exists(os.path.join(path, 'filters', 'FILTER.RES.latest')):
        fetch_eazy_photoz()
        
    return path


def test_read_filter_res():
    """
    Read FILTER.RES
    """
    data_path = test_data_path()
    filter_file = os.path.join(data_path, 'filters/FILTER.RES.latest')
    res = filters.FilterFile(filter_file)
    
    assert(res[155].name.startswith('REST_FRAME/maiz-apellaniz_Johnson_V'))
    assert(np.allclose(res[155].pivot, 5479.35, rtol=0.001))
    return res


def go():
    
    import numpy as np
    import glob
    import matplotlib.pyplot as plt
    
    from astropy.io import fits
    import eazy

    zsp_only = False

    # GOODS-S
    self = eazy.photoz.PhotoZ(param_file='zphot.param.goodss.uvista', translate_file='zphot.translate.goodss.uvista', zeropoint_file='zphot.zeropoint.goodss.uvista.FSPS')
    
    self = eazy.photoz.PhotoZ(param_file='zphot.param.new', translate_file='zphot.translate.new', zeropoint_file='zphot.zeropoint.new')
    
    # UVUDF
    self = eazy.photoz.PhotoZ(param_file='zphot.param', translate_file='zphot.translate', zeropoint_file='zphot.zeropoint')
    use = (self.cat['STAR'] == 0) 
    
    # R-selected
    self = eazy.photoz.PhotoZ(param_file='zphot.param.new', translate_file='zphot.translate.new', zeropoint_file='zphot.zeropoint.FSPS')
    
    # 
    self.zp = self.zp*0+1
    
    if False:
        out = fits.open('eazy_output.fits')
        self.zp = out['ZP'].data
        self.zbest = out['ZBEST'].data
        self.chibest = out['CHIBEST'].data
        self.fit_chi2 = out['FIT_CHI2'].data
        self.fobs = out['FOBS'].data
        
        for i in range(self.NFILT):
            self.cat['fit_{0}'.format(self.flux_columns[i])] = self.fobs[:,i]/self.ext_corr[i]
            
    # mag = self.param['PRIOR_ABZP'] - 2.5*np.log10(self.cat['Ks'])
    # 
    # mag = self.param['PRIOR_ABZP'] - 2.5*np.log10(self.cat['rp_tot'])

    zp_ix = self.f_numbers == self.param['PRIOR_FILTER']
    mag = self.param['PRIOR_ABZP'] - np.squeeze(2.5*np.log10(self.fnu[:, zp_ix]))
    
    idx = (mag < 23) | ((self.cat['z_spec'] > 0) & np.isfinite(mag))
    #idx = (mag < 22) | ((self.cat['z_spec'] > 0) & np.isfinite(mag))

    if zsp_only:
        idx &= (self.cat['z_spec'] > 0)
    
    use = (self.cat['use'] == 1) & (self.cat['star_flag'] != 1)
    
    if False:
        # UltraVISTA
        use = (self.cat['USE'] == 1) & (self.cat['star'] == 0)
        use = (self.cat['contamination'] == 0) & (self.cat['nan_contam'] == 0) & (self.cat['star'] == 0)
    
    idx &= use
    
    idx = np.arange(self.NOBJ)[idx]
    
    # test
    iter = 100
    error_residuals, update_templates, update_zeropoints = False, False, False
    
    self.iterate_zp_templates(idx=idx, update_templates=update_templates, iter=iter, error_residuals=error_residuals, update_zeropoints=update_zeropoints, save_templates=False, n_proc=4)
    
    rf_tempfilt, f_rest = self.rest_frame_fluxes(f_numbers=[153,155,161], pad_width=0.1, percentiles=[2.5,16,50,84,97.5])
    
    uv = -2.5*np.log10(f_rest[:,0,2]/f_rest[:,1,2])
    vj = -2.5*np.log10(f_rest[:,1,2]/f_rest[:,2,2])
    
    import astropy.cosmology
    from astropy.cosmology import Planck15
    cosmo = Planck15
    cosmo = astropy.cosmology.FlatLambdaCDM(H0=70, Om0=0.3)
    
    import astropy.units as u
    
    dL = cosmo.luminosity_distance(self.zgrid).to(u.cm)
    dL_i = np.interp(self.zbest, self.zgrid, dL)

    #mass_ratio = (np.interp(self.zbest, self.zgrid, dL)/np.interp(fout['z'], self.zgrid, dL))**2
    
    fnu_factor = 10**(-0.4*(self.param['PRIOR_ABZP']+48.6))
    Lnu = (f_rest[:,:,2].T*fnu_factor*4*np.pi*dL_i**2/(1+self.zbest)).T
    nuLnu = Lnu*(3.e8/(rf_tempfilt.lc*1.e-10))
    Ltot = nuLnu / 3.839e33
    Lv = Ltot[:,1]
    Lj = Ltot[:,2]
    
    lines = open('templates/uvista_nmf/spectra_kc13_12_tweak.param').readlines()
    MLv_template = np.asarray([line.split()[3] for line in lines if ('MLv' in line) & (line.startswith('# '))],dtype=float)
    SFRv_template = np.asarray([line.split()[5] for line in lines if ('MLv' in line) & (line.startswith('# '))],dtype=float)

    irest = 1 # V
    #irest = 2 # J
    
    ### Full individual template set
    if 'fsps_full' in self.param['TEMPLATES_FILE']:
        nu = 3.e18/rf_tempfilt.lc[irest]
        Lvt = rf_tempfilt.tempfilt[:,irest]*nu # nu Lnu luminosity

        # These are masses
        lines = open(self.param['TEMPLATES_FILE']).readlines()
        MLv_template = np.asarray([line.split()[4] for line in lines if line.startswith('# ')],dtype=float)
        SFRv_template = np.asarray([line.split()[5] for line in lines if line.startswith('# ')],dtype=float)

        MLv_template /= Lvt
        SFRv_template /= Lvt

    csum = self.coeffs_best*rf_tempfilt.tempfilt[:,irest]
    csum = (csum.T/f_rest[:,irest,2]).T
    #csum = (csum.T/self.coeffs_best.sum(axis=1)).T
    MLv = np.sum(csum*MLv_template, axis=1)#/np.sum(csum, axis=1)
    stellar_mass = Ltot[:,irest]*MLv
    sfr = Ltot[:,irest]*np.sum(csum*SFRv_template, axis=1)#/np.sum(csum, axis=1)
    
    tf = rf_tempfilt.tempfilt
    uv_temp = -2.5*np.log10(tf[:,0]/tf[:,1])
    vj_temp = -2.5*np.log10(tf[:,1]/tf[:,2])
    
    plt.scatter(vj[idx], uv[idx], alpha=0.1,c=np.log10(MLv)[idx], vmin=-1, vmax=1.5, edgecolor='None')
    plt.scatter(vj_temp, uv_temp, alpha=0.8,c=np.log10(MLv_template), vmin=-1, vmax=1.5, marker='s', edgecolor='w', s=30)
    plt.xlim(-0.5,3); plt.ylim(-0.5,3)

    # sSFR
    ssfr = sfr / stellar_mass
    plt.scatter(vj[idx], uv[idx], alpha=0.1,c=np.log10(ssfr)[idx], vmin=-13, vmax=-8, edgecolor='None', cmap='jet_r')
    #plt.scatter(vj_temp, uv_temp, alpha=0.8,c=np.log10(SFRv_template/MLv_template), vmin=-13, vmax=-10, marker='s', edgecolor='w', s=30)
    plt.xlim(-0.5,3); plt.ylim(-0.5,3)
    
    j = 0
    
    mass_fraction = ((csum*MLv_template).T/(csum*MLv_template).sum(axis=1)).T
    # L fraction
    light_fraction = ((csum).T/(csum).sum(axis=1)).T
    
    weight = mass_fraction
    
    j+=1; id = self.cat['id'][idx[j]]; fig = self.show_fit(id, show_fnu=show_fnu, xlim=[0.1,10])

    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.scatter(vj[idx], uv[idx], alpha=0.1,c=np.log10(MLv)[idx], vmin=-1, vmax=1.5, edgecolor='None')
    ax.scatter(vj_temp, uv_temp, c=np.log10(MLv_template), vmin=-1, vmax=1.5, marker='s', edgecolor='k', s=200*weight[idx[j],:]**0.3)
    ax.scatter(vj[idx][j], uv[idx][j], alpha=0.8,c=np.log10(MLv)[idx][j], vmin=-1, vmax=1.5, edgecolor='k', s=70)
    ax.set_xlim(-0.5,5); ax.set_ylim(-0.5,5)
    
    fout = catIO.Table('/Users/brammer/3DHST/Spectra/Release/v4.1.5/FullRelease/goodss_3dhst_v4.1.5_catalogs/goodss_3dhst.v4.1.5.zbest.fout')
    
    # run
    iter = -1
    
    for iter in range(7):
        self.iterate_zp_templates(idx=idx, update_templates=(iter > 1), iter=iter, error_residuals=(iter > 0), save_templates=(iter > 8), n_proc=4)
        self.zp[-3] = 1.0
        self.write_zeropoint_file('zphot.zeropoint.FSPS')
    
    # Show fits
    if False:
        ids = self.cat['id'][idx]
        j = -1

        j+=1; id = ids[j]; fig = self.show_fit(id, show_fnu=show_fnu, xlim=[0.1,10])
     
    ## Full Run
    idx = np.arange(self.NOBJ)[use]
    
    self.fit_parallel(idx=idx, n_proc=4)
    self.best_fit()
    self.error_residuals()
    
    self.fit_parallel(idx=idx, n_proc=4)
    self.best_fit()
    rf_tempfilt, f_rest = self.rest_frame_fluxes(f_numbers=[153,155,161], pad_width=0.5, percentiles=[2.5,16,50,84,97.5])
    
    hdu = fits.HDUList()
    hdu.append(fits.ImageHDU(data=self.zp, name='zp'))
    hdu.append(fits.ImageHDU(data=self.zbest, name='zbest'))
    hdu.append(fits.ImageHDU(data=self.chi_best, name='chibest'))
    hdu.append(fits.ImageHDU(data=self.fit_chi2, name='fit_chi2'))
    hdu.append(fits.ImageHDU(data=self.fobs, name='fobs'))
    hdu.writeto('eazy_output.fits', clobber=True)
    
    np.save('eazy_templates.npy', [self.templates])
    
    ## NUV mag
    idx = np.arange(self.NOBJ)[use & (mag < 23) & (self.zbest < 1)]
    #plt.scatter(nuv_obs[idx]*self.zp[-1]*self.ext_corr[-1], nuv_mod[idx], alpha=0.1)
    #plt.xlim(0.01,100); plt.ylim(0.01, 100); plt.plot([0.01,100], [0.01,100], color='r'); plt.loglog()
    
    plt.hist(25-2.5*np.log10(nuv_obs[idx]*self.zp[-1]), bins=100, range=[18,28], alpha=0.5, log=True)
    plt.hist(25-2.5*np.log10(nuv_mod[idx]/self.ext_corr[-1]), bins=100, range=[18,28], alpha=0.5, log=True)
    
    nuv_mag = 25-2.5*np.log10(nuv_mod/self.ext_corr[-1]*totcorr)
    idx = use & (nuv_mag < 22) & (self.zbest > 0.8)
    
    